package org.livee.data.batch;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.db.CommandFactory;
import org.livee.data.db.DatabaseAdapter;
import org.livee.data.db.GetProfileAllCommand;
import org.livee.data.db.SetAdminProfileCommand;
import org.livee.data.db.InvalidateOldAdminProfileCommand;
import org.livee.data.expr.AbstractData;

import org.livee.data.tools.LogManager;
import org.livee.data.tools.NeighborManager;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class DataManagerProfileMergeBatch implements AbstractBatch {

	int m_Interval;
	int m_CacheValidityTime;

	public DataManagerProfileMergeBatch(){
		org.livee.data.tools.ConfigManager config=org.livee.data.tools.ConfigManager.getInstance();
		m_Interval=config.getIntParameter("/c:liveeConfiguration/c:master/c:dataManagerProfileMerger/c:interval/text()");
		if(m_Interval<=0){
			m_Interval=1200;
		}
		m_CacheValidityTime=config.getIntParameter("/c:liveeConfiguration/c:master/c:dataManagerProfileMerger/c:cacheValidityTime/text()");
		if(m_CacheValidityTime<=0){
			m_CacheValidityTime=7200;
		}

	}
	
	public int interval(){
		return m_Interval;
	}
	
	public void execute() throws Exception {

		LogManager.postNormal("master.dataManagerProfileManager.begin", null);
		
		try{
			AbstractData profile=null;

			NeighborManager neighbor=NeighborManager.getInstance();
			GetProfileAllCommand getCommand=CommandFactory.createGetProfileAllCommand();
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			
			InvalidateOldAdminProfileCommand invalidateCommand=new InvalidateOldAdminProfileCommand(m_CacheValidityTime);

			db.postCommand(invalidateCommand);
			db.postCommand(getCommand);
			
			if(getCommand.succeeded()){
				profile=getCommand.getResult();
			}else{
				// Error while loading profile from datamanager
				LogManager.postError("master.dataManagerProfileManager.error", "While loading profile from datamanager");		
			}
			String site=neighbor.getThisSiteName();
		
			if(profile!=null){
				SetAdminProfileCommand command=CommandFactory.createSetAdminProfileCommand(site, profile);
				db.postCommand(command);
				
				if(!command.succeeded()){
					// Error while setting adminProfile
					LogManager.postError("master.dataManagerProfileManager.error", "While setting adminProfile");
				}
			}
		}
		catch(Exception e){
			// Error
			LogManager.postError("master.dataManagerProfileManager.error", e.getMessage());
			return ;
		}
		LogManager.postNormal("master.dataManagerProfileManager.end", null);
	}

	public String getBatchName() {
		return "DataManagerProfileMerge";
	}

}
