package org.livee.data.batch;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.db.DatabaseAdapter;
import org.livee.data.db.GetProfileAllCommand;
import org.livee.data.expr.AbstractData;

import org.livee.data.tools.LogManager;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class ProfileCheckBatch implements AbstractBatch {

	int m_Interval;
	String m_OutputLevel;
	
	public ProfileCheckBatch(){
		org.livee.data.tools.ConfigManager config=org.livee.data.tools.ConfigManager.getInstance();
		m_Interval=config.getIntParameter("/c:liveeConfiguration/c:master/c:profileChecker/c:interval/text()");
		if(m_Interval<=0){
			m_Interval=600;
		}
		m_OutputLevel=config.getStringParameter("/c:liveeConfiguration/c:master/c:profileChecker/c:errOutputLevel/text()");
		if(m_OutputLevel==null){
			m_OutputLevel="error";
		}else{
			if(!m_OutputLevel.matches("null|normal|warning|error")){
				m_OutputLevel="error";
			}
		}
	}
	
	public int interval(){
		return m_Interval;
	}
	
	public void execute() throws Exception {

		LogManager.postNormal("master.profileChecker.begin", null);
		
		if(m_OutputLevel.equals("null")){
			LogManager.postNormal("master.profileChecker.end", null);
			return ;
		}
		
		try{
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			GetProfileAllCommand command=new GetProfileAllCommand(); 
		
			db.postCommand(command);
		
			if(command.succeeded()){
				AbstractData collection=command.getResult();
				org.livee.data.tools.ProfileSchemaProcessor schema=org.livee.data.tools.ProfileSchemaProcessor.getInstance();
			
				java.util.Collection children=collection.getChildren();
				if(children!=null){
					java.util.Iterator itr=children.iterator();
					while(itr.hasNext()){
						AbstractData combined=(AbstractData)itr.next();
						String[] warnings=schema.validateAttribute(combined);
					
						if(warnings!=null){
							if(m_OutputLevel.equals("normal")){
								for(int i=0;i<warnings.length;i++){
									LogManager.postNormal("master.profileChecker.message", warnings[i]);
								}
							}else if(m_OutputLevel.equals("warning")){
								for(int i=0;i<warnings.length;i++){
									LogManager.postWarning("master.profileChecker.warning", warnings[i]);
								}
							}else if(m_OutputLevel.equals("error")){
								for(int i=0;i<warnings.length;i++){
									LogManager.postError("master.profileChecker.error", warnings[i]);
								}								
							}
						}
					}
				}
			}
		}
		catch(Exception e){
			LogManager.postError("master.profileChecker.error", e.getMessage());
			return ;
		}
		
		LogManager.postNormal("master.profileChecker.end", null);
	}

	public String getBatchName() {
		return "ProfileCheck";
	}

}
