package org.livee.data.batch;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.expr.AbstractData;
import org.livee.data.tools.LogManager;
import org.livee.data.tools.NeighborManager;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class ProfileSchemaLoadBatch implements AbstractBatch {

	int m_Interval;
	
	public ProfileSchemaLoadBatch(){		
		org.livee.data.tools.ConfigManager config=org.livee.data.tools.ConfigManager.getInstance();
		m_Interval=config.getIntParameter("/c:liveeConfiguration/c:master/c:profileSchemaLoader/c:interval/text()");
		if(m_Interval<=0){
			m_Interval=3600;
		}
	}
	
	public int interval(){
		return m_Interval;
	}
	
	public void execute() throws Exception {

		LogManager.postNormal("master.profileSchemaLoader.begin", null);
		
		boolean b_succeeded=false;
		String schema=null;
		
		NeighborManager neighbor=NeighborManager.getInstance();
		
		AbstractData adminSite=neighbor.getParentAdminSite();
		if(adminSite!=null){

			java.util.Iterator itr=adminSite.getChildren().iterator();
			while(itr.hasNext() && !b_succeeded){
				AbstractData adminServer=(AbstractData)itr.next();
				String service=adminServer.getAttribute("service");
				String url=adminServer.getAttribute("url");
				String linkStatus=adminServer.getAttribute("linkStatus");
				
				if(service.equals("Admin200703") && (linkStatus==null || linkStatus.equals("up"))){
					
					try{
						org.livee.data.soap.client.Admin200703Service srv=new org.livee.data.soap.client.Admin200703ServiceLocator();
						org.livee.data.soap.client.Admin200703_PortType admin=srv.getAdmin200703(new java.net.URL(url));
						schema=admin.getSchema();
					}
					catch(Exception e){
						// Warning: An error occured while accessing the service.
						LogManager.postWarning("master.profileSchemaLoader.warning", "While accessing a remote service: "+e.getMessage());
						continue ;						
					}

					if(schema!=null){ 	
						// 1. Try to parse the schema;
						org.livee.data.expr.AbstractData adminSchema=null;
						org.w3c.dom.Document doc=org.livee.util.Xml.parseXml(schema);
						if(doc==null){
							// Warning: throw new Exception("An error occured while loading and parsing the specified schema file.");
							LogManager.postWarning("master.profileSchemaLoader.warning", "XML parse failure");
							continue ;
						}
						try{
							org.w3c.dom.Element xml=doc.getDocumentElement();	
							org.livee.data.expr.ParserFactory pf=org.livee.data.expr.ParserFactory.getInstance();
							org.livee.data.expr.AbstractParser adminSchema_parser=pf.get("adminSchema");
							
							adminSchema=adminSchema_parser.parse(xml);
						}
						catch(Exception e){
							// Warning 
							LogManager.postWarning("master.profileSchemaLoader.warning", "While parsing a schema: "+e.getMessage());
							continue;
						}
					
						// 2. Save the got schema.
						try{
							org.livee.data.db.SetAdminSchemaCommand command=org.livee.data.db.CommandFactory.createSetAdminSchemaCommand(adminSchema);
							org.livee.data.db.DatabaseAdapter db=org.livee.data.db.DatabaseAdapter.getInstance();
							db.postCommand(command);
							
							if(!command.succeeded()){
								// Warning
								LogManager.postWarning("master.profileSchemaLoader.warning", "Failed to save the gotton schema");
								continue;
							}else{
								b_succeeded=true;
							}
						}
						catch(Exception e){
							// e.printStackTrace();
							// Warning
							LogManager.postWarning("master.profileSchemaLoader.warning", e.getMessage());
						}
					}
				}
			}
		}
		
		if(b_succeeded && adminSite!=null){
			LogManager.postNormal("master.profileSchemaLoader.end", null);
		}else{
			// Error
			LogManager.postError("master.profileSchemaLoader.error", "Failed to get the schema");
		}
	}

	public String getBatchName() {
		return "ProfileSchemaLoad";
	}

}
