package org.livee.data.batch;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.command.bk.AdministratorOLD.CheckStoppedSensorCommand;
import org.livee.data.db.DatabaseAdapter;

import org.livee.data.tools.LogManager;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class SensorLivingCheckBatch implements AbstractBatch {

	
	public class CheckStoppedSensorCommand implements org.livee.data.db.IDbCommand{
		java.util.Collection m_Collection;

		public CheckStoppedSensorCommand(){
			m_Collection=new java.util.ArrayList();
		}

		boolean m_Succeeded;

		public boolean succeeded() {
			return m_Succeeded;
		}

		public java.util.Collection getResult() {
			return m_Collection;
		}

		public String getCommandName() {
			return "CheckStoppedSensor";
		}

		public short getImplVersion() {
			return 0x0100;
		}

		public void execCommand(Connection connection, Map conf) throws Exception {
			java.sql.Statement stmt=null;
			try {
				stmt = connection.createStatement();
				
				java.util.Calendar cal=java.util.Calendar.getInstance();
				if(m_ThresholdTimeBase.equals("day")){
					cal.add(java.util.Calendar.DATE, -m_ThresholdTime);
				}else if(m_ThresholdTimeBase.equals("hour")){
					cal.add(java.util.Calendar.HOUR, -m_ThresholdTime);
				}else if(m_ThresholdTimeBase.equals("minute")){
					cal.add(java.util.Calendar.MINUTE, -m_ThresholdTime);
				}
				
				String sql = "SELECT id,ownerName,ownerMailAddress FROM sensor.combined WHERE id IN "+
					"(SELECT sensor.element.combined_id AS id FROM raw.latest, sensor.element WHERE raw.latest.id=sensor.element.id AND time<\'"+org.livee.util.W3CTimestamp.toString(cal)+"\')"+   // Updated 2008-10-01 by Hideya
					"AND working=\'t\' ORDER BY id";
				
				java.sql.ResultSet rset = stmt.executeQuery(sql);
				while (rset.next()) {
					String id=rset.getString("id");
					String ownerName=rset.getString("ownerName");
					String ownerMailAddress=rset.getString("ownerMailAddress");

					java.util.Hashtable table=new java.util.Hashtable();
					table.put("id", id);
					table.put("ownerName", ownerName);
					table.put("ownerMailAddress",ownerMailAddress);
					m_Collection.add(table);
				} 
				rset.close();
				m_Succeeded = true;
			} catch (Exception e) {
			
			}finally{
				stmt.close();				
			}
		}
	}
	
	int m_Interval;
	int m_ThresholdTime;
	String m_ThresholdTimeBase;
	String m_OutputLevel;
	
	public SensorLivingCheckBatch(){
		org.livee.data.tools.ConfigManager config=org.livee.data.tools.ConfigManager.getInstance();
		m_Interval=config.getIntParameter("/c:liveeConfiguration/c:master/c:sensorLivingChecker/c:interval/text()");
		m_ThresholdTime=config.getIntParameter("/c:liveeConfiguration/c:master/c:sensorLivingChecker/c:thresholdTime/text()");
		m_ThresholdTimeBase=config.getStringParameter("/c:liveeConfiguration/c:master/c:sensorLivingChecker/c:thresholdTimeBase/text()");
		m_OutputLevel=config.getStringParameter("/c:liveeConfiguration/c:master/c:sensorLivingChecker/c:errOutputLevel/text()");
				
		if(m_Interval<=0){
			m_Interval=600;
		}
		if(m_ThresholdTime<=0){
			m_ThresholdTime=3;
		}
		if(m_ThresholdTimeBase==null || !m_ThresholdTimeBase.matches("minute|hour|day")){
			m_ThresholdTimeBase="day";
		}
		if(m_OutputLevel==null){
			m_OutputLevel="error";
		}else{
			if(!m_OutputLevel.matches("null|normal|warning|error")){
				m_OutputLevel="error";
			}
		}
	}
	
	public int interval(){
		return m_Interval;
	}
	
	public void execute() throws Exception {

		LogManager.postNormal("master.sensorLivingChecker.begin", null);
		
		if(m_OutputLevel.equals("null")){
			LogManager.postNormal("master.sensorLivingChecker.end", null);
			return ;
		}
		
		try{
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			CheckStoppedSensorCommand command=new CheckStoppedSensorCommand(); 
		
			db.postCommand(command);
			
			if(command.succeeded()){
				java.util.Collection collection=command.getResult();
				
				java.util.Iterator itr=collection.iterator();
				while(itr.hasNext()){
					java.util.Map combined=(java.util.Map)itr.next();
					
					String id=(String)combined.get("id");
					String ownerName=(String)combined.get("ownerName");
					if(m_OutputLevel.equals("normal")){
						LogManager.postNormal("master.sensorLivingChecker.message", ownerName+"\'s sensor ("+id+") is stopped..");
					}else if(m_OutputLevel.equals("warning")){
						LogManager.postWarning("master.sensorLivingChecker.warning", ownerName+"\'s sensor ("+id+") is stopped..");
					}else if(m_OutputLevel.equals("error")){
						LogManager.postError("master.sensorLivingChecker.error", ownerName+"\'s sensor ("+id+") is stopped..");
					}
				}
			}
		}catch(Exception e){
			// Error
			LogManager.postError("master.sensorLivingChecker.error", e.getMessage());
		}

		LogManager.postNormal("master.sensorLivingChecker.end", null);

	}

	public String getBatchName() {
		return "SensorLivingCheck";
	}

}
