package org.livee.data.command;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.batch.*;
import org.livee.data.expr.AbstractData;
import org.livee.data.expr.AbstractParser;
import org.livee.data.tools.ConfigManager;
import org.livee.data.tools.LogManager;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import java.util.ArrayList;
import java.util.Collection;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class Administrator {

	private static Collection m_BatchCol;
	
	public static void main(String[] args) {
		
		ConfigManager conf=ConfigManager.getInstance();
		String mode=conf.getStringParameter("/c:liveeConfiguration/c:mode/text()");
		
		if(mode==null && !mode.matches("master|slave")){
			mode="master";
		}
		
		m_BatchCol=new ArrayList();
		
		if(mode.equals("master")){
			initBatch();
			
			m_BatchCol.add(new AggregationBatch());
			m_BatchCol.add(new ChildProfileMergeBatch());
			m_BatchCol.add(new DataManagerProfileMergeBatch());
			m_BatchCol.add(new ProfileCheckBatch());
			m_BatchCol.add(new ProfileSchemaLoadBatch());
			m_BatchCol.add(new SensorLivingCheckBatch());
		}else if(mode.equals("slave")){
			m_BatchCol.add(new CopyAdminBatch());
			m_BatchCol.add(new CopyProfileBatch());			
			m_BatchCol.add(new CopyArchiveDataBatch());
			m_BatchCol.add(new CopyLatestDataBatch());
		}
		
		long count=0;
		while(true){
			try{
				java.util.Iterator itr=m_BatchCol.iterator();
				while(itr.hasNext()){
					AbstractBatch batch=(AbstractBatch)itr.next();
					int interval=batch.interval();
					if((count%interval)==0){
						try{  batch.execute(); }catch(Exception e){}
					}
				}
				Thread.sleep(1000);
			}catch(Exception e){
				e.printStackTrace();
			}
			count++;
		}
	}
	
	private static void initBatch(){
		// Master should load "topology" and "schema" if it is given
		
		// topology -- NeighborManager
		LogManager.postNormal("master.init.topologyLoader.begin", null);
		
		ConfigManager config=ConfigManager.getInstance();
		Node neighbor_node=config.getNodeParameter("/c:liveeConfiguration/c:neighbor/c:topology/a:neighbor");
		if(neighbor_node!=null){
			try{
				Element xml=(Element)neighbor_node;
				AbstractParser parser=org.livee.data.expr.ParserFactory.getInstance().get("adminNeighbor");
				AbstractData neighbor=parser.parse(xml);
				org.livee.data.tools.NeighborManager nbr=org.livee.data.tools.NeighborManager.getInstance();
				nbr.setNeighbor(neighbor);
			}catch(Exception e){
				LogManager.postError("master.init.topologyLoader.error", e.getMessage());
			}
			LogManager.postNormal("master.init.topologyLoader.end", null);
		}else{
			LogManager.postError("master.init.topologyLoader.error", "Failed to load neighbor topology information.");
		}
		
		// schema -- copy ProfileSchemaLoader
		LogManager.postNormal("master.init.profileSchemaLoader.begin", null);
		Node schema_node=config.getNodeParameter("/c:liveeConfiguration/c:profileSchema/c:schema/a:adminSchema");
		if(schema_node!=null){
			try{
				AbstractParser parser=org.livee.data.expr.ParserFactory.getInstance().get("adminSchema");			
				AbstractData profileSchema=null;
				profileSchema=parser.parse((Element)schema_node);

				org.livee.data.db.SetAdminSchemaCommand command=org.livee.data.db.CommandFactory.createSetAdminSchemaCommand(profileSchema);
				org.livee.data.db.DatabaseAdapter db=org.livee.data.db.DatabaseAdapter.getInstance();
				db.postCommand(command);
				
				if(command.succeeded()){
					LogManager.postNormal("master.init.profileSchemaLoader.message", "profileSchema Loaded.");
				}else{
					LogManager.postError("master.init.profileSchemaLoader.error", "Failed to load schema though it is given.");
				}
			}
			catch(Exception e){
				// e.printStackTrace();
				LogManager.postError("master.init.profileSchemaLoader.error", e.getMessage());
			}
		}
		LogManager.postNormal("master.init.profileSchemaLoader.end", null);
		
		try{
			Thread.sleep(1000);
		}
		catch(Exception e){}
	}
}
