package org.livee.data.command;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.sql.Connection;
import java.util.Map;

import org.livee.data.expr.AbstractData;
import org.livee.data.db.*;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class Authorization {

	public class GetSensorListCommand implements org.livee.data.db.IDbCommand {
		java.util.Collection m_Collection;

		public GetSensorListCommand(){
			m_Collection=new java.util.ArrayList();
		}

		boolean m_Succeeded;

		public boolean succeeded() {
			return m_Succeeded;
		}

		public java.util.Collection getResult() {
			return m_Collection;
		}

		public String getCommandName() {
			return "GetSensorList";
		}

		public short getImplVersion() {
			return 0x0100;
		}

		public void execCommand(Connection connection, Map conf) throws Exception {
			java.sql.Statement stmt=null;
			try {
				stmt = connection.createStatement();
				String sql = "SELECT id,passwd,working,ownerName,ownerMailAddress FROM sensor.combined ORDER BY id";
				
				java.sql.ResultSet rset = stmt.executeQuery(sql);
				while (rset.next()) {
					String id=rset.getString("id");
					String passwd=rset.getString("passwd");
					boolean working=rset.getBoolean("working");
					String ownerName=rset.getString("ownerName");
					String ownerMailAddress=rset.getString("ownerMailAddress");

					if(working){
						m_Collection.add(id+", "+passwd+", working, "+ownerName+", "+ownerMailAddress);
					}else{
						m_Collection.add(id+", "+passwd+",  closed, "+ownerName+", "+ownerMailAddress);
					}
				} 
				rset.close();
				m_Succeeded = true;
			} catch (Exception e) {
			
			}finally{
				stmt.close();				
			}
		}

	}
	
	
	private String m_Command=null;
	private String m_ID=null;
	private String m_Passwd=null;
	private String m_OwnerName=null;
	private String m_OwnerMailAddress=null;
	
	private boolean parseArgs(String[] args){
		boolean bRet=false;
		
		if(args.length>=1){
			m_Command=args[0];
			if(m_Command.equals("list") || m_Command.equals("add") || m_Command.equals("update") || m_Command.equals("close") || m_Command.equals("restore")){
				int i=1;
				while(i<args.length-1){
					if(args[i].equals("-i")){
						m_ID=args[i+1];
					}else if(args[i].equals("-p")){
						m_Passwd=args[i+1];
					}else if(args[i].equals("-o")){
						m_OwnerName=args[i+1];
					}else if(args[i].equals("-m")){
						m_OwnerMailAddress=args[i+1];
					}
					i+=2;
				}
				bRet=true;
			}
		}
		
		if(!bRet){
			printUsage();
		}		
		return bRet;
	}
	
	private void printUsage(){
		System.out.println("Usage: java org.livee.data.command.Authorization COMMAND [OPTIONS]");
		System.out.println();
		System.out.println("COMMAND:= list | add | update | close | restore");
		System.out.println("OPTIONS:=(OPTION)* | e");
		System.out.println("OPTION:= -i ID | -p PASSWORD | -o OWNER_NAME | -m OWNER_MAIL_ADDRESS");
	}
	
	private void showAuthorizationList(){
		
		try {
			DatabaseAdapter db = DatabaseAdapter.getInstance();
			
			GetSensorListCommand command = new GetSensorListCommand();
			
			db.postCommand(command);

			java.util.Collection collection=command.getResult();
			java.util.Iterator itr=collection.iterator();
			
			int count=0;
			while(itr.hasNext()){
				String combined=(String)itr.next();
				System.out.println(combined);
				count++;
			}
			System.out.println();
			System.out.println(count+" sensors are registered now...");
		}
		catch(Exception e){
			//e.printStackTrace();
		}
	}
	
	private void addAuthorization() throws IOException {
		while(m_ID==null || "".equals(m_ID)){
			System.out.print("new ID: ");
			BufferedReader reader=new BufferedReader(new InputStreamReader(System.in));
			m_ID=reader.readLine();
		}
		
		while(m_Passwd==null || "".equals(m_Passwd)){
			System.out.print("Password: ");
			BufferedReader reader=new BufferedReader(new InputStreamReader(System.in));
			String pass0=reader.readLine();
			
			System.out.print("Retype-Password: ");
			String pass1=reader.readLine();
			
			if((pass0==null && pass1==null) || (pass0!=null && pass0.equals(pass1))){
				m_Passwd=pass0;
			}
		}
		
		while(m_OwnerName==null || "".equals(m_OwnerName)){
			System.out.print("Owner Name: ");
			BufferedReader reader=new BufferedReader(new InputStreamReader(System.in));
			m_OwnerName=reader.readLine();
		}
		
		while(m_OwnerMailAddress==null || "".equals(m_OwnerMailAddress)){
			System.out.print("Owner Mail Address: ");
			BufferedReader reader=new BufferedReader(new InputStreamReader(System.in));
			m_OwnerMailAddress=reader.readLine();
		}
		
		
		AuthorizeSensorCommand command=null;
		try {
			DatabaseAdapter db = DatabaseAdapter.getInstance();
			
			command= CommandFactory.createAuthorizeSensorCommand(m_ID,m_Passwd,m_OwnerName,m_OwnerMailAddress);
			db.postCommand(command);

		}
		catch(Exception e){
			//e.printStackTrace();
		}

		if(command!=null && command.succeeded()){
			System.out.println("Succeeded...");
		}else{
			System.out.println("Failed.. The id might exist already. len(id)<10 or len(pass)<6");
		}
		
	}
	
	private void updateAuthorizationPassword() throws Exception{
		while(m_ID==null || "".equals(m_ID)){
			System.out.print("update ID: ");
			BufferedReader reader=new BufferedReader(new InputStreamReader(System.in));
			m_ID=reader.readLine();			
		}
		
		/*
		while(m_Passwd==null || "".equals(m_Passwd)){
			System.out.print("Password: ");
			BufferedReader reader=new BufferedReader(new InputStreamReader(System.in));
			String pass0=reader.readLine();
			
			System.out.print("Retype-Password: ");
			String pass1=reader.readLine();
			
			if((pass0==null && pass1==null) || (pass0!=null && pass0.equals(pass1))){
				m_Passwd=pass0;
			}
		}

		while(m_OwnerName==null || "".equals(m_OwnerName)){
			System.out.print("Owner Name: ");
			BufferedReader reader=new BufferedReader(new InputStreamReader(System.in));
			m_OwnerName=reader.readLine();
		}
		
		while(m_OwnerMailAddress==null || "".equals(m_OwnerMailAddress)){
			System.out.print("Owner Mail Address: ");
			BufferedReader reader=new BufferedReader(new InputStreamReader(System.in));
			m_OwnerMailAddress=reader.readLine();
		}
        */
		
		UpdateAuthorizedSensorCommand command=null;
		try {
			DatabaseAdapter db = DatabaseAdapter.getInstance();
			
			command = CommandFactory.createUpdateAuthorizedSensorCommand(m_ID,m_Passwd,m_OwnerName,m_OwnerMailAddress);
			db.postCommand(command);

		}
		catch(Exception e){
			//e.printStackTrace();
		}

		if(command!=null && command.succeeded()){
			System.out.println("Succeeded...");
		}else{
			System.out.println("Failed.. The id might be incorrect. or len(id)<10 or len(pass)<6");
		}
	}
	
	private void closeSensor() throws Exception {
		while(m_ID==null || "".equals(m_ID)){
			System.out.print("close sensor ID: ");
			BufferedReader reader=new BufferedReader(new InputStreamReader(System.in));
			m_ID=reader.readLine();			
		}
		
		try{
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			CloseCombinedCommand command=new CloseCombinedCommand(m_ID,null);
			db.postCommand(command);
			if(command.succeeded()){
				System.out.println("Sensor: "+m_ID+ " was closed..");
			}else{
				System.out.println("Failed.. "+m_ID+ " might not have been registered..");
			}
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	private void restoreSensor() throws Exception {
		while(m_ID==null || "".equals(m_ID)){
			System.out.print("restore sensor ID: ");
			BufferedReader reader=new BufferedReader(new InputStreamReader(System.in));
			m_ID=reader.readLine();			
		}
		
		try{
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			RestoreCombinedCommand command=new RestoreCombinedCommand(m_ID,null);
			db.postCommand(command);
			if(command.succeeded()){
				System.out.println("Sensor: "+m_ID+ " is now restored..");
			}else{
				System.out.println("Failed.. "+m_ID+ " might not have been registered..");
			}
		}catch(Exception e){
			e.printStackTrace();
		}		
	}
	
	private void exec(String[] args) throws Exception {
		if(parseArgs(args)){
			if(m_Command.equals("list")){
				showAuthorizationList();
			}else if(m_Command.equals("add")){
				addAuthorization();
			}else if(m_Command.equals("update")){
				updateAuthorizationPassword();
			}else if(m_Command.equals("close")){
				closeSensor();
			}else if(m_Command.equals("restore")){
				restoreSensor();
			}
		}		
	}
	
	public static void main(String[] args) throws Exception {
		
		// debugging
		//String[] myargs={"restore"};
		//args=myargs;
		
		Authorization auth=new Authorization();
		auth.exec(args);
	}

}
