package org.livee.data.command;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.srv.DataProvider200703;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class LocalDataRetriever {

	String m_Command;
	String m_ID;
	String m_Start;
	String m_End;
	String m_Locale;
	String m_TimeZone;
	
	public boolean parseArgs(String[] args){
		boolean bRet=false;
		if(args.length>=1){
			m_Command=args[0];
			if(m_Command.matches("getProfileSchema|getLatestDataAll|getLatestData|getDataHourlyAggregated|getDataDailyAggregated|getDataMonthlyAggregated|getProfileAll|getProfile")){				
				int i=1;
				m_Locale="English";
				m_TimeZone=java.util.TimeZone.getDefault().getID();
				while(i<args.length-1){
					if(args[i].equals("-i")){
						m_ID=args[i+1];
					}else if(args[i].equals("-s")){
						m_Start=args[i+1];
					}else if(args[i].equals("-e")){
						m_End=args[i+1];
					}else if(args[i].equals("-l")){
						m_Locale=args[i+1];
					}else if(args[i].equals("-tz")){
						m_TimeZone=args[i+1];
					}
					i+=2;
				}
				bRet=true;
			}
		}
		
		if(!bRet){
			printUsage();
		}
		
		return bRet;
	}
	
	private void printUsage(){
		System.out.println("Usage: java org.livee.data.command.LocalDataRetriever COMMAND [OPTIONS]");
		System.out.println();
		System.out.println("COMMAND:= getProfileSchema | getLatestDataAll ");
		System.out.println("        | getLatestData | getDataHourlyAggregated ");
		System.out.println("        | getDataDailyAggregated | getDataMonthlyAggregated");
		System.out.println("        | getProfileAll | getProfile"); 
		System.out.println("OPTIONS:=(OPTION)* | e");
		System.out.println("OPTION:= -i ID | -s START_TIMESTAMP | -e END_TIMESTAMP");
		System.out.println("       | -l LOCALE | -tz TIMEZONE");
	}


	private void execute(String[] args) throws Exception {
		if(parseArgs(args)){
			DataProvider200703 provider=new DataProvider200703();
			if(m_Command.equals("getProfileSchema")){
				System.out.println(provider.getProfileSchema());
			}else if(m_Command.equals("getLatestDataAll")){
				System.out.println(provider.getLatestDataAll(m_TimeZone, m_Locale));
			}else if(m_Command.equals("getLatestData")){
				if(m_ID==null){
					System.out.println("Error: getLatestData needs -i option."); 
					return ;
				}
				System.out.println(provider.getLatestData(m_ID, m_TimeZone, m_Locale));
			}else if(m_Command.equals("getDataHourlyAggregated")){
				if(m_ID==null || m_Start==null || m_End==null){
					System.out.println("Error: getHourlyAggregated needs -i,-s and -e options."); 
					return ;
				}
				System.out.println(provider.getDataHourlyAggregated(m_ID, m_Start,m_End, m_TimeZone, m_Locale));
			}else if(m_Command.equals("getDataDailyAggregated")){
				if(m_ID==null || m_Start==null || m_End==null){
					System.out.println("Error: getDailyAggregated needs -i,-s and -e options."); 
					return ;
				}
				System.out.println(provider.getDataDailyAggregated(m_ID, m_Start,m_End, m_TimeZone, m_Locale));
			}else if(m_Command.equals("getDataMonthlyAggregated")){
				if(m_ID==null || m_Start==null || m_End==null){
					System.out.println("Error: getMonthlyAggregated needs -i,-s and -e options."); 
					return ;
				}
				System.out.println(provider.getDataMonthlyAggregated(m_ID, m_Start,m_End, m_TimeZone, m_Locale));
			}else if(m_Command.equals("getProfileAll")){
				System.out.println(provider.getProfileAll(m_TimeZone, m_Locale));
			}else if(m_Command.equals("getProfile")){
				if(m_ID==null){
					System.out.println("Error: getProfile needs -i options."); 
					return ;
				}
				System.out.println(provider.getProfile(m_ID, m_TimeZone, m_Locale));
			}
		}
	}
	
	public static void main(String[] args) throws Exception {
		LocalDataRetriever resolver=new LocalDataRetriever();
		resolver.execute(args);
		System.exit(0);
	}
}
