package org.livee.data.command;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.io.BufferedReader;
import java.io.InputStreamReader;
import org.livee.util.Xml;

import org.livee.data.srv.ProfileManagement200703;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class ProfileManager {

	String m_Command;
	String m_ID;
	String m_Passwd;
	String m_FilePath;
	
	private boolean parseArgs(String[] args){
		boolean bRet=false;
		
		if(args.length>=1){
			m_Command=args[0];
			if(m_Command.equals("list") || m_Command.equals("get") || m_Command.equals("set") || m_Command.equals("schema")){
				int i=1;
				while(i<args.length-1){
					if(args[i].equals("-i")){
						m_ID=args[i+1];
					}else if(args[i].equals("-p")){
						m_Passwd=args[i+1];
					}else if(args[i].equals("-f")){
						m_FilePath=args[i+1];
					}
					i+=2;
				}
				bRet=true;
			}
		}
		
		if(!bRet){
			printUsage();
		}		
		return bRet;
	}
	
	private void printUsage(){
		System.out.println("Usage: java org.livee.data.command.ProfileManager COMMAND [OPTIONS]");
		System.out.println();
		System.out.println("COMMAND:= list | get | set | schema");
		System.out.println("OPTIONS:=(OPTION)* | e");
		System.out.println("OPTION:= -i ID | -p PASSWORD | -f FILEPATH");
	}
	
	private void showSensorList(){
		ProfileManagement200703 pm=new ProfileManagement200703();
		System.out.println(pm.getProfileAll(java.util.TimeZone.getDefault().getID(), "English"));
	}
	
	private void getProfile() throws java.io.IOException {
		
		while(m_ID==null || "".equals(m_ID)){
			System.out.print("ID: ");
			BufferedReader reader=new BufferedReader(new InputStreamReader(System.in));
			m_ID=reader.readLine();
		}
		
		ProfileManagement200703 pm=new ProfileManagement200703();
		System.out.println(pm.getProfile(m_ID));		
	}
	
	private void setProfile() throws java.io.IOException {

		while(m_FilePath==null || "".equals(m_FilePath)){
			System.out.print("FilePath: ");
			BufferedReader reader=new BufferedReader(new InputStreamReader(System.in));
			m_FilePath=reader.readLine();
		}
		
		while(m_Passwd==null || "".equals(m_Passwd)){
			System.out.print("Password: ");
			BufferedReader reader=new BufferedReader(new InputStreamReader(System.in));
			String pass0=reader.readLine();
			
			System.out.print("Retype-Password: ");
			String pass1=reader.readLine();
			
			if((pass0==null && pass1==null) || (pass0!=null && pass0.equals(pass1))){
				m_Passwd=pass0;
			}
		}
		
		org.w3c.dom.Document doc=Xml.parseXmlFile(m_FilePath);
		if(doc==null){
			System.err.println("Failed to load: "+m_FilePath);
			System.err.println("File may not be existed or the Xml is broken.");
			return ;
		}
		
		org.w3c.dom.Element root=doc.getDocumentElement();
		
		root.setAttribute("authorization", org.livee.util.Base64Coder.encodeString(m_Passwd));
		ProfileManagement200703 pm=new ProfileManagement200703();
		System.out.println(pm.setProfile(Xml.toString(root)));
			
	}
	
	private void getProfileSchema(){
		ProfileManagement200703 pm=new ProfileManagement200703();
		System.out.println(pm.getProfileSchema());		
	}

	private void exec(String[] args) throws Exception {
		if(parseArgs(args)){
			if(m_Command.equals("list")){
				showSensorList();
			}else if(m_Command.equals("get")){
				getProfile();
			}else if(m_Command.equals("set")){
				setProfile();
			}else if(m_Command.equals("schema")){
				getProfileSchema();
			}
		}		
	}
	
	public static void main(String[] args) throws Exception {
		ProfileManager pm=new ProfileManager();
		pm.exec(args);
		System.exit(0);
	}

}
