package org.livee.data.command.bk;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/**
 * ProfileLoader is obsolete
 * This class is for debugging
 * @author Hideya Ochiai, The University of Tokyo
 * @see org.livee.data.command.Administrator
 * Created: before 2008-01-01
 */
public class AdminProfileLoader {

	private static void printUsage(){
		System.out.println("Usage: java org.livee.data.command.ProfileLoader COMMAND FIELD");
		System.out.println("<<COMMAND>>");
		System.out.println("list          -- displays system profile schema");
		System.out.println("load filename targetSite    -- load system locale schema (from localfile)");		
		System.out.println("remote url targetSite       -- load system locale schema (from remote Admin200703 service)");
		
		System.out.println();
	}
	
	private static void loadRemoteProfile(String url,String targetSite) throws Exception{

		org.livee.data.soap.client.Admin200703Service srv=new org.livee.data.soap.client.Admin200703ServiceLocator();
		org.livee.data.soap.client.Admin200703_PortType profile=srv.getAdmin200703(new java.net.URL(url));
		
		String xml=profile.getProfileAll();
		
		org.w3c.dom.Document doc=org.livee.util.Xml.parseXml(xml);
		if(doc==null){
			System.err.println("An error occured while loading and parsing the specified xml file.");
			return ;
		}
		
		org.w3c.dom.Element root=doc.getDocumentElement();
		
		org.livee.data.expr.ParserFactory pf=org.livee.data.expr.ParserFactory.getInstance();
		org.livee.data.expr.AbstractParser collection_parser=pf.get("collection");
		
		org.livee.data.expr.AbstractData collection=null;
		try{
			collection=collection_parser.parse(root);
		}
		catch(Exception e){
			System.err.println("An error occured while parsing the profile document.");
			e.printStackTrace();
			return ;
		}
		
		try{
			org.livee.data.db.SetAdminProfileCommand command=org.livee.data.db.CommandFactory.createSetAdminProfileCommand(targetSite,collection);
			org.livee.data.db.DatabaseAdapter db=org.livee.data.db.DatabaseAdapter.getInstance();
			db.postCommand(command);
			
			if(command.succeeded()){
				System.out.println("Succeeded...");
			}else{
				System.err.println("Failed to load...");
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	private static void loadProfile(String filepath,String targetSite){
		
		org.w3c.dom.Document doc=org.livee.util.Xml.parseXmlFile(filepath);
		if(doc==null){
			System.err.println("An error occured while loading and parsing the specified xml file.");
			return ;
		}
		
		org.w3c.dom.Element root=doc.getDocumentElement();
		
		org.livee.data.expr.ParserFactory pf=org.livee.data.expr.ParserFactory.getInstance();
		org.livee.data.expr.AbstractParser collection_parser=pf.get("collection");
		
		org.livee.data.expr.AbstractData collection=null;
		try{
			collection=collection_parser.parse(root);
		}
		catch(Exception e){
			System.err.println("An error occured while parsing the profile document.");
			e.printStackTrace();
			return ;
		}
		
		try{
			org.livee.data.db.SetAdminProfileCommand command=org.livee.data.db.CommandFactory.createSetAdminProfileCommand(targetSite,collection);
			org.livee.data.db.DatabaseAdapter db=org.livee.data.db.DatabaseAdapter.getInstance();
			db.postCommand(command);
			
			if(command.succeeded()){
				System.out.println("Succeeded...");
			}else{
				System.err.println("Failed to load...");
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	private static void showProfile(){

		try{
			org.livee.data.db.GetAdminProfileCommand command=org.livee.data.db.CommandFactory.createGetAdminProfileCommand();
			org.livee.data.db.DatabaseAdapter db=org.livee.data.db.DatabaseAdapter.getInstance();
			db.postCommand(command);
			
			if(command.succeeded()){
				org.livee.data.expr.AbstractData collection=command.getCollection();
				if(collection!=null){
					org.w3c.dom.Document doc=org.livee.util.Xml.newDocument();
					org.w3c.dom.Element root=collection.getXml(doc);
					System.out.println(org.livee.util.Xml.toString(root));
					return ;
				}
			}
			System.err.println("Failed...");
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	public static void main(String[] args){
			
		if(args.length==1){
			if(args[0].equals("list")){
				showProfile();
			}else{
				printUsage();
			}
		}else if(args.length==3){
			if(args[0].equals("load")){
				loadProfile(args[1],args[2]);
			}else if(args[0].equals("remote")){
				
				try{
					loadRemoteProfile(args[1],args[2]);
				}
				catch(Exception e){
					e.printStackTrace();
				}
			}else{
				printUsage();
			}
			
		}else{
			printUsage();
		}
		
	}
}
