package org.livee.data.command.bk;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import org.livee.data.expr.AbstractParser;
import org.livee.data.expr.AbstractData;
import org.livee.data.expr.ParserFactory;
import org.livee.data.expr.AdminNeighborData;

import org.livee.data.db.*;

import org.livee.util.Xml;
import org.livee.util.W3CTimestamp;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.sql.Connection;
import java.util.Calendar;
import java.util.Map;


/**
 * AdministratorOLD is obsolete
 * @author Hideya Ochiai, The University of Tokyo
 * @see org.livee.data.command.Administrator
 * Created: before 2008-01-01
 */
public class AdministratorOLD {

	public class CheckStoppedSensorCommand implements org.livee.data.db.IDbCommand{
		java.util.Collection m_Collection;

		public CheckStoppedSensorCommand(){
			m_Collection=new java.util.ArrayList();
		}

		boolean m_Succeeded;

		public boolean succeeded() {
			return m_Succeeded;
		}

		public java.util.Collection getResult() {
			return m_Collection;
		}

		public String getCommandName() {
			return "CheckStoppedSensor";
		}

		public short getImplVersion() {
			return 0x0100;
		}

		public void execCommand(Connection connection, Map conf) throws Exception {
			java.sql.Statement stmt=null;
			try {
				stmt = connection.createStatement();
				
				java.util.Calendar cal=java.util.Calendar.getInstance();
				cal.add(java.util.Calendar.DATE, -1);
				
				String sql = "SELECT id,ownerName,ownerMailAddress FROM sensor.combined WHERE id IN "+
					"(SELECT sensor.element.combined_id AS id FROM raw.latest, sensor.element WHERE raw.latest.id=sensor.element.id AND time<\'"+cal.getTime().toString()+"\')"+
					"AND working=\'t\' ORDER BY id";
				
				java.sql.ResultSet rset = stmt.executeQuery(sql);
				while (rset.next()) {
					String id=rset.getString("id");
					String ownerName=rset.getString("ownerName");
					String ownerMailAddress=rset.getString("ownerMailAddress");

					java.util.Hashtable table=new java.util.Hashtable();
					table.put("id", id);
					table.put("ownerName", ownerName);
					table.put("ownerMailAddress",ownerMailAddress);
					m_Collection.add(table);
				} 
				rset.close();
				m_Succeeded = true;
			} catch (Exception e) {
			
			}finally{
				stmt.close();				
			}
		}
	}
	
	
	
	boolean m_bMaster;
	AbstractData m_AdminConf;
	AbstractData m_AdminNeighbor;
	boolean m_bAdminNeighborUpdated;
	String m_MasterURL;
	String m_MasterDataManagerURL;
	String m_Site;
	
	/**
	 * getProfileAll - GetProfile from children
	 * @throws Exception
	 */
	private void getProfileAll() throws Exception {

		String failed_sites="";
		java.util.Iterator itr=m_AdminNeighbor.getChildren().iterator();
		SiteLoop : while(itr.hasNext()){
			AbstractData adminSite=(AbstractData)itr.next();
			String site=adminSite.getAttribute("name");
			String type=adminSite.getAttribute("type");
			if(type.equals("child")){
				java.util.Iterator itr2=adminSite.getChildren().iterator();
				boolean b_succeeded=false;
				while(itr2.hasNext() && !b_succeeded){
					AbstractData adminServer=(AbstractData)itr2.next();
					String service=adminServer.getAttribute("service");
					String url=adminServer.getAttribute("url");
					String linkStatus=adminServer.getAttribute("linkStatus");

					if(service.equals("Admin200703")){
						String profileAll=null;
						boolean linkAlive=false;
						try{
							org.livee.data.soap.client.Admin200703Service srv=new org.livee.data.soap.client.Admin200703ServiceLocator();
							org.livee.data.soap.client.Admin200703_PortType admin=srv.getAdmin200703(new java.net.URL(url));
							profileAll=admin.getProfileAll();
							//Document doc=Xml.parseXmlFile("latestDataAll.xml");
							//latestDataAll=Xml.toString(doc.getDocumentElement());
							linkAlive=true;
						}
						catch(Exception e){
							linkAlive=false;
						}
						// Link information update
						if(linkAlive && (linkStatus==null || !linkStatus.equals("up"))){
							adminServer.setAttribute("linkStatus","up");
							m_bAdminNeighborUpdated=true;
						}
						if(!linkAlive && (linkStatus==null || !linkStatus.equals("down"))){
							adminServer.setAttribute("linkStatus", "down");
							m_bAdminNeighborUpdated=true;
						}
						
						if(profileAll!=null){
							// 1. Try to parse the latestDataAll;
							org.w3c.dom.Document doc=org.livee.util.Xml.parseXml(profileAll);
							if(doc==null){
								// throw new Exception("An error occured while loading and parsing the data");
								failed_sites+=site+", ";
								continue SiteLoop;
							}
							
							org.w3c.dom.Element xml=doc.getDocumentElement();	
							org.livee.data.expr.ParserFactory pf=org.livee.data.expr.ParserFactory.getInstance();
							org.livee.data.expr.AbstractParser sensorCollection_parser=pf.get("collection");
							
							org.livee.data.expr.AbstractData sensorCollection=null;
							try{
								sensorCollection=sensorCollection_parser.parse(xml);
							}
							catch(Exception e){
								e.printStackTrace();
								// throw new Exception("An error occured while parsing the schema.");
								failed_sites+=site+", ";
								continue SiteLoop;
							}
							
							// 2. Save the got data;
							try{
								org.livee.data.db.SetAdminProfileCommand command=org.livee.data.db.CommandFactory.createSetAdminProfileCommand(site,sensorCollection);
								org.livee.data.db.DatabaseAdapter db=org.livee.data.db.DatabaseAdapter.getInstance();
								db.postCommand(command);
								
								if(!command.succeeded()){
									// throw new Exception("Failed to save the got data");
									failed_sites+=site+", ";
									continue SiteLoop;
								}
								b_succeeded=true;
							}
							catch(Exception e){
								e.printStackTrace();
								// throw e;
								failed_sites+=site+", ";
								continue SiteLoop;
							}

						}
					}
					if(!b_succeeded){
						failed_sites+=site+", ";
					}
				}
			}
		}

		if(failed_sites.length()>=1){
			throw new Exception("Failed to get latest data from site: "+failed_sites);
		}
	}
	
	/**
	 * GetSchema from the parent
	 * @throws Exception
	 */
	private void getSchema() throws Exception {
		
		boolean b_succeeded=false;
		boolean parent_certainly_exists=false;
		String schema=null;

		java.util.Collection children=m_AdminNeighbor.getChildren();
		if(children.size()==0){
			return ;
		}
		
		java.util.Iterator itr=children.iterator();
		while(itr.hasNext() && !b_succeeded){
			AbstractData adminSite=(AbstractData)itr.next();
			String type=adminSite.getAttribute("type");
			if(type.equals("parent")){
				parent_certainly_exists=true;
				
				java.util.Iterator itr2=adminSite.getChildren().iterator();
				while(itr2.hasNext() && !b_succeeded){
					AbstractData adminServer=(AbstractData)itr2.next();
					String service=adminServer.getAttribute("service");
					String url=adminServer.getAttribute("url");
					String linkStatus=adminServer.getAttribute("linkStatus");
						
					if(service.equals("Admin200703")){
						
						boolean linkAlive=false;
						try{
							org.livee.data.soap.client.Admin200703Service srv=new org.livee.data.soap.client.Admin200703ServiceLocator();
							org.livee.data.soap.client.Admin200703_PortType admin=srv.getAdmin200703(new java.net.URL(url));
							schema=admin.getSchema();
							linkAlive=true;
						}
						catch(Exception e){
							linkAlive=false;
						}
						// Link information update
						if(linkAlive && (linkStatus==null || !linkStatus.equals("up"))){
							adminServer.setAttribute("linkStatus","up");
							m_bAdminNeighborUpdated=true;
						}
						if(!linkAlive && (linkStatus==null || !linkStatus.equals("down"))){
							adminServer.setAttribute("linkStatus", "down");
							m_bAdminNeighborUpdated=true;
						}
						
						if(schema!=null){ 
							
							// 1. Try to parse the schema;
							org.w3c.dom.Document doc=org.livee.util.Xml.parseXml(schema);
							if(doc==null){
								throw new Exception("An error occured while loading and parsing the specified schema file.");
							}
							org.w3c.dom.Element xml=doc.getDocumentElement();	
							org.livee.data.expr.ParserFactory pf=org.livee.data.expr.ParserFactory.getInstance();
							org.livee.data.expr.AbstractParser adminSchema_parser=pf.get("adminSchema");
							
							org.livee.data.expr.AbstractData adminSchema=null;
							try{
								adminSchema=adminSchema_parser.parse(xml);
							}
							catch(Exception e){
								e.printStackTrace();
								throw new Exception("An error occured while parsing the schema.");
							}
								// 2. Save the got schema.
							try{
								org.livee.data.db.SetAdminSchemaCommand command=org.livee.data.db.CommandFactory.createSetAdminSchemaCommand(adminSchema);
								org.livee.data.db.DatabaseAdapter db=org.livee.data.db.DatabaseAdapter.getInstance();
								db.postCommand(command);
								
								if(!command.succeeded()){
									throw new Exception("Failed to save the got schema");
								}
								b_succeeded=true;
							}
							catch(Exception e){
								e.printStackTrace();
								throw e;
							}
						}
					}
				}
			}		
		}

		if(parent_certainly_exists && !b_succeeded){
			throw new Exception("Failed to get schema..");
		}	
	}
	
	private void loadDataManager() throws Exception {
		
		GetProfileAllCommand getCommand=CommandFactory.createGetProfileAllCommand();
		GetAdminNeighborCommand neighbor=CommandFactory.createGetAdminNeighborCommand();
	
		DatabaseAdapter db=DatabaseAdapter.getInstance();
		
		String site=null;
		AbstractData profile=null;
		
		try{
			db.postCommand(getCommand);
			if(getCommand.succeeded()){
				profile=getCommand.getResult();
			}
			
			db.postCommand(neighbor);
			if(neighbor.succeeded()){
				AbstractData sites=neighbor.getAdminNeighbor();
				java.util.Iterator itr=sites.getChildren().iterator();
				while(itr.hasNext()){
					AbstractData adminSite=(AbstractData)itr.next();
					if(adminSite.getAttribute("type").equals("colleague")){
						site=adminSite.getAttribute("name");
						break;
					}
				}
			}
			
			if(profile!=null){
				SetAdminProfileCommand command=CommandFactory.createSetAdminProfileCommand(site, profile);
				db.postCommand(command);
				
				if(command.succeeded()){
					System.out.println("Admin Updated...");
				}
			}
		}
		catch(Exception e){
			
		}
		
	}
	
	private void copyMaster() throws Exception {
		
		if(!m_bMaster && m_MasterURL!=null){
			String replicated_xml=null;
			try{
				org.livee.data.soap.client.Admin200703Service srv=new org.livee.data.soap.client.Admin200703ServiceLocator();
				org.livee.data.soap.client.Admin200703_PortType admin=srv.getAdmin200703(new java.net.URL(m_MasterURL));			
				replicated_xml=admin.replicate();
				//System.out.println(replicated_xml);
			}
			catch(Exception e){
				e.printStackTrace();
			}
			
			if(replicated_xml==null){
				throw new Exception("Failed to get master's replication..");
			}
			
			try{
				Document doc=Xml.parseXml(replicated_xml);
				Element root=doc.getDocumentElement();
				
			//	Document doc=Xml.parseXmlFile("admin.db.xml");
			//	Element root=doc.getDocumentElement();
				
				ParserFactory pf=ParserFactory.getInstance();
				AbstractParser adminReplicate_parser=pf.get("adminReplicate");
				AbstractData adminReplicate=adminReplicate_parser.parse(root);
				
				DatabaseAdapter db=DatabaseAdapter.getInstance();
				RestoreAdminCommand command=CommandFactory.createRestoreAdminCommand(adminReplicate);
				db.postCommand(command);
				
				if(command.succeeded()){
					System.out.println("Master's replication was successfully loaded..");
				}else{
					throw new Exception("Failed to load master's replication..");
				}

			}
			catch(Exception e){
				e.printStackTrace();
				throw e;
			}

		}	
	}
	
	private void replicateLatestDataManager(){
		
		try{
			org.livee.data.soap.client.DataManagerReplicationService drps=new org.livee.data.soap.client.DataManagerReplicationServiceLocator();
			org.livee.data.soap.client.DataManagerReplication srv=drps.getDataManagerReplication200703(new java.net.URL(m_MasterDataManagerURL));
			
			String dataManager_string=srv.replicateLatestData();
			System.out.println(dataManager_string);
			
			Document doc=Xml.parseXml(dataManager_string);
			Element dataManager_xml=doc.getDocumentElement();
			
			AbstractParser dm_parser=org.livee.data.replicate.ParserFactory.getInstance().get("replicateDataManager");
			AbstractData dataManager=dm_parser.parse(dataManager_xml);
			
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			
			org.livee.data.replicate.RestoreLatestDataCommand command=new org.livee.data.replicate.RestoreLatestDataCommand(dataManager);
			db.postCommand(command);
			
			if(command.succeeded()){
				System.out.println("replicateLatestData succeeded..");
			}
			
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	private void replicateNextArchiveDataManager(){
		
		try{
			java.util.Calendar comparator=null;
			java.util.Calendar from=null;
			java.util.Calendar to=null;
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			
			org.livee.data.replicate.GetSlaveReplicationProfileCommand gscommand=new org.livee.data.replicate.GetSlaveReplicationProfileCommand("dataArchive");
			db.postCommand(gscommand);
			
			if(gscommand.succeeded()){
				from=gscommand.getDone();
			}else{
				from=java.util.Calendar.getInstance();
				from.set(2007, 0, 1,0,0,0);
			}
			comparator=java.util.Calendar.getInstance();
			comparator.add(Calendar.MINUTE, -300);
			if(from.after(comparator)){
				return ;
				//from=comparator;
			}
			to=java.util.Calendar.getInstance();
			to.setTimeInMillis(from.getTimeInMillis());
			to.add(Calendar.MINUTE, 290);
			
			
			org.livee.data.soap.client.DataManagerReplicationService drps=new org.livee.data.soap.client.DataManagerReplicationServiceLocator();
			org.livee.data.soap.client.DataManagerReplication srv=drps.getDataManagerReplication200703(new java.net.URL(m_MasterDataManagerURL));
			
			String dataManager_string=srv.replicateArchiveData(W3CTimestamp.toString(from), W3CTimestamp.toString(to));
			System.out.println(dataManager_string);
			
			Document doc=Xml.parseXml(dataManager_string);
			Element dataManager_xml=doc.getDocumentElement();
			
			AbstractParser dm_parser=org.livee.data.replicate.ParserFactory.getInstance().get("replicateDataManager");
			AbstractData dataManager=dm_parser.parse(dataManager_xml);
						
			org.livee.data.replicate.RestoreArchiveDataCommand command=new org.livee.data.replicate.RestoreArchiveDataCommand(dataManager);
			db.postCommand(command);
			
			if(command.succeeded()){
				
				org.livee.data.replicate.SetSlaveReplicationProfileCommand sscommand=new org.livee.data.replicate.SetSlaveReplicationProfileCommand("dataArchive",to);
				db.postCommand(sscommand);
				
				if(sscommand.succeeded()){
					System.out.println("archiveReplication succeeded...");
				}
			}
			
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
		
	}
	
	private void replicateNextProfileDataManager(){
		
		try{
			java.util.Calendar comparator=null;
			java.util.Calendar from=null;
			java.util.Calendar to=null;
			DatabaseAdapter db=DatabaseAdapter.getInstance();
			
			org.livee.data.replicate.GetSlaveReplicationProfileCommand gscommand=new org.livee.data.replicate.GetSlaveReplicationProfileCommand("profileArchive");
			db.postCommand(gscommand);
			
			if(gscommand.succeeded()){
				from=gscommand.getDone();
			}else{
				from=java.util.Calendar.getInstance();
				from.set(2007, 0, 1,0,0,0);
			}
			comparator=java.util.Calendar.getInstance();
			comparator.add(Calendar.MINUTE, -300);
			if(from.after(comparator)){
				return ;
				//from=comparator;
			}
			to=java.util.Calendar.getInstance();
			to.setTimeInMillis(from.getTimeInMillis());
			to.add(Calendar.MINUTE, 290);
			
			
			org.livee.data.soap.client.DataManagerReplicationService drps=new org.livee.data.soap.client.DataManagerReplicationServiceLocator();
			org.livee.data.soap.client.DataManagerReplication srv=drps.getDataManagerReplication200703(new java.net.URL(m_MasterDataManagerURL));
			
			String dataManager_string=srv.replicateProfile(W3CTimestamp.toString(from), W3CTimestamp.toString(to));
			System.out.println(dataManager_string);
			
			Document doc=Xml.parseXml(dataManager_string);
			Element dataManager_xml=doc.getDocumentElement();
			
			AbstractParser dm_parser=org.livee.data.replicate.ParserFactory.getInstance().get("replicateDataManager");
			AbstractData dataManager=dm_parser.parse(dataManager_xml);
						
			org.livee.data.replicate.RestoreProfileCommand command=new org.livee.data.replicate.RestoreProfileCommand(dataManager);
			db.postCommand(command);
			
			if(command.succeeded()){
				
				org.livee.data.replicate.SetSlaveReplicationProfileCommand sscommand=new org.livee.data.replicate.SetSlaveReplicationProfileCommand("profileArchive",to);
				db.postCommand(sscommand);
				
				if(sscommand.succeeded()){
					System.out.println("profileReplication succeeded...");
				}
			}
			
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	private void checkNeighbor(){  // check Naming consistency and linkStatus 
		
		java.util.Iterator itr=m_AdminNeighbor.getChildren().iterator();
		while(itr.hasNext()){
			AbstractData adminSite=(AbstractData)itr.next();
			String site=adminSite.getAttribute("name");
			String type=adminSite.getAttribute("type");

			if(type.equals("parent")){
				java.util.Iterator itr2=adminSite.getChildren().iterator();
				while(itr2.hasNext()){
					AbstractData adminServer=(AbstractData)itr2.next();
					String service=adminServer.getAttribute("service");
					String url=adminServer.getAttribute("url");
					String linkStatus=adminServer.getAttribute("linkStatus");
						
					if(service.equals("Admin200703")){
						String neighbor=null;
						boolean linkAlive=false;
						try{
							org.livee.data.soap.client.Admin200703Service srv=new org.livee.data.soap.client.Admin200703ServiceLocator();
							org.livee.data.soap.client.Admin200703_PortType admin=srv.getAdmin200703(new java.net.URL(url));
							neighbor=admin.getNeighbor();
							
							linkAlive=true;
						}
						catch(Exception e){
							linkAlive=false;
						}
						// Link information update
						if(linkAlive && (linkStatus==null || !linkStatus.equals("up"))){
							adminServer.setAttribute("linkStatus","up");
							m_bAdminNeighborUpdated=true;
						}
						if(!linkAlive && (linkStatus==null || !linkStatus.equals("down"))){
							adminServer.setAttribute("linkStatus", "down");
							m_bAdminNeighborUpdated=true;
						}

						if(neighbor!=null){
							try{
								Document doc=Xml.parseXml(neighbor);
								Element xml=doc.getDocumentElement();
								ParserFactory pf=ParserFactory.getInstance();
								AbstractData adminNeighbor=pf.get("adminNeighbor").parse(xml);
								int c=AdminNeighborData.getRelation(m_AdminNeighbor,adminNeighbor);
								if(c!=AdminNeighborData.NEIGHBOR_PARENT){
									System.err.println("Inconsistency neighbor site: "+site);
								}
							}catch(Exception e){
								e.printStackTrace();
							}
						}
					}
				}
				
			}else if(type.equals("child")){
				java.util.Iterator itr2=adminSite.getChildren().iterator();
				while(itr2.hasNext()){
					AbstractData adminServer=(AbstractData)itr2.next();
					String service=adminServer.getAttribute("service");
					String url=adminServer.getAttribute("url");
					String linkStatus=adminServer.getAttribute("linkStatus");
						
					if(service.equals("Admin200703")){
						
						String neighbor=null;
						boolean linkAlive=false;
						try{
							org.livee.data.soap.client.Admin200703Service srv=new org.livee.data.soap.client.Admin200703ServiceLocator();
							org.livee.data.soap.client.Admin200703_PortType admin=srv.getAdmin200703(new java.net.URL(url));
							neighbor=admin.getNeighbor();
							
							linkAlive=true;
						}
						catch(Exception e){
							linkAlive=false;
						}
						// Link information update
						if(linkAlive && (linkStatus==null || !linkStatus.equals("up"))){
							adminServer.setAttribute("linkStatus","up");
							m_bAdminNeighborUpdated=true;
						}
						if(!linkAlive && (linkStatus==null || !linkStatus.equals("down"))){
							adminServer.setAttribute("linkStatus", "down");
							m_bAdminNeighborUpdated=true;
						}

						if(neighbor!=null){
							try{
								Document doc=Xml.parseXml(neighbor);
								Element xml=doc.getDocumentElement();							
								ParserFactory pf=ParserFactory.getInstance();
								AbstractData adminNeighbor=pf.get("adminNeighbor").parse(xml);
								int c=AdminNeighborData.getRelation(m_AdminNeighbor,adminNeighbor);
								if(c!=AdminNeighborData.NEIGHBOR_CHILD){
									System.err.println("Inconsistency neighbor site: "+site);
								}
							}
							catch(Exception e){
								e.printStackTrace();
							}
						}

					}
				}
			}
			
			
		}
	}
	
	private void setConf(AbstractData adminConfig) throws Exception {
		
		DatabaseAdapter db=DatabaseAdapter.getInstance();
		SetAdminConfigCommand command=CommandFactory.createSetAdminConfigCommand(adminConfig);
		db.postCommand(command);
			
		if(command.succeeded()){
			System.out.println("Configuration was successfully loaded...");
		}else{
			throw new Exception("configuration error...");
		}
	}
	
	private void parseConf(AbstractData adminConfig){
		
		java.util.Iterator itr=adminConfig.getChildren().iterator();
		while(itr.hasNext()){
			AbstractData data=(AbstractData)itr.next();
			if(data.getDataType().equals("AdminConfigParameter")){
				AbstractData param=data;
				String name=param.getAttribute("name");
				String value=param.getAttribute("value");
			
				if(name.equals("mode")){
					if(value.equals("master")){
						m_bMaster=true;
					}else if(value.equals("slave")){
						m_bMaster=false;
					}else{
						System.err.println("Invalid mode:"+value);
					}
				}else if(name.equals("masterURL")){
					if(value.startsWith("http://")){
						m_MasterURL=value;
					}else{
						System.err.println("Invalid master URL:"+value);
					}
				}else if(name.equals("masterDataManagerURL")){
					if(value.startsWith("http://")){
						m_MasterDataManagerURL=value;
					}else{
						System.err.println("Invalid masterDataManager URL:"+value);
					}
					
				}else if(name.equals("site")){
					m_Site=value;	
				}
			}else if(data.getDataType().equals("AdminNeighbor")){
				m_AdminNeighbor=data;
			}
		}		
	}
	
	private void aggregateData() throws Exception {
		DatabaseAdapter db=DatabaseAdapter.getInstance();
		AggregationCommand command=CommandFactory.createAggregationCommand();
		
		db.postCommand(command);
		
		if(command.succeeded()){
			System.out.println("Aggregation: Succeeded...");
		}
	}
	
	private void profileSchemaChecker() throws Exception {
		DatabaseAdapter db=DatabaseAdapter.getInstance();
		GetProfileAllCommand command=new GetProfileAllCommand(); 
		
		db.postCommand(command);
		
		if(command.succeeded()){
			AbstractData collection=command.getResult();
			org.livee.data.tools.ProfileSchemaProcessor schema=org.livee.data.tools.ProfileSchemaProcessor.getInstance();
			
			java.util.Collection children=collection.getChildren();
			if(children!=null){
				java.util.Iterator itr=children.iterator();
				while(itr.hasNext()){
					AbstractData combined=(AbstractData)itr.next();
					String[] warnings=schema.validateAttribute(combined);
					
					if(warnings!=null){
						for(int i=0;i<warnings.length;i++){
							System.err.println(warnings[i]);
						}
					}
				}
			}
		}
	}
	
	private void checkStoppedSensor() throws Exception {

		DatabaseAdapter db=DatabaseAdapter.getInstance();
		CheckStoppedSensorCommand command=new CheckStoppedSensorCommand(); 
		
		db.postCommand(command);
		
		if(command.succeeded()){
			java.util.Collection collection=command.getResult();
			
			java.util.Iterator itr=collection.iterator();
			while(itr.hasNext()){
				java.util.Map combined=(java.util.Map)itr.next();
				
				System.out.println("id: "+combined.get("id"));
				System.out.println("ownerName: "+combined.get("ownerName"));
				System.out.println("ownerMailAddress: "+combined.get("ownerMailAddress"));
				System.out.println();
			}
		}
		
	}
	
	private AbstractData loadConf(String filepath) throws Exception {
	
		Document doc=Xml.parseXmlFile(filepath);
		Element root=doc.getDocumentElement();
		
		ParserFactory pf=ParserFactory.getInstance();
		AbstractParser adminConfig_parser=pf.get("adminConfig");
		
		AbstractData adminConfig=adminConfig_parser.parse(root);
		return adminConfig;		
	}
	
	private void execute(){
		
		try{
			AbstractData adminConfig=loadConf("admin.conf.xml");
			parseConf(adminConfig);
			
			if(m_bMaster){
				setConf(adminConfig);
			}
			
			long loop=0;
			while(true){
				try{
					if(m_bMaster){
						if((loop%60)==0){
							aggregateData();
						}

						if((loop%3600)==0){
							getSchema();
							//try{ getSchema(); }catch(Exception e){}
						}
						if((loop%1800)==1){
							getProfileAll();
							//try{ getProfileAll(); }catch(Exception e){}
						}
						if((loop%60)==0){
							checkNeighbor();
							//try{ checkNeighbor(); }catch(Exception e){}
						}
						if((loop%55)==0){
							loadDataManager();
							//try{ loadDataManager(); }catch(Exception e){}
						}
						if((loop%50)==0){
							profileSchemaChecker();
						}
						if((loop%3600)==5){
							checkStoppedSensor();
						}
					}else{
						if((loop%60)==5){
							replicateLatestDataManager();
						}
//						if((loop%1)==0){
							replicateNextProfileDataManager();
	//					}
		//				if((loop%1)==0){
							replicateNextArchiveDataManager();
			//			}
						
						if((loop%600)==0){
							copyMaster();
						}
					}
					Thread.sleep(1000);
				}
				catch(Exception e){
					e.printStackTrace();
				}
				loop++;
			}
			
		}
		catch(Exception e){
			e.printStackTrace();
		}

	}
	
	public static void main(String[] args) {
		AdministratorOLD process=new AdministratorOLD();
		process.execute();	
	}

}
