package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.expr.AbstractData;
import org.livee.data.expr.DataFactory;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class DumpAdminCommand implements IDbCommand {
	
	private boolean m_Succeeded;
	private AbstractData m_AdminReplicate;
	
	protected DumpAdminCommand(){
		m_Succeeded=false;
	}
	
	public boolean succeeded(){
		return m_Succeeded;
	}
	
	public AbstractData getReplicatedAdmin(){
		return m_AdminReplicate;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
	
		DataFactory df=DataFactory.getInstance();
		m_AdminReplicate=df.create("adminReplicate");
		
		GetAdminConfigCommand adminConfig_command=new GetAdminConfigCommand();
		//GetAdminNeighborCommand adminNeighbor_command=new GetAdminNeighborCommand();
		GetAdminLocaleSchemaCommand adminLocaleSchema_command=new GetAdminLocaleSchemaCommand();
		GetAdminProfileSchemaCommand adminProfileSchema_command=new GetAdminProfileSchemaCommand();
		GetAdminProfileReplicateCommand adminProfile_command=new GetAdminProfileReplicateCommand();		
		
		try{
			m_Succeeded=true;

			adminConfig_command.execCommand(connection, conf);
			if(adminConfig_command.succeeded()){
				m_AdminReplicate.append(adminConfig_command.getAdminConfig());
			}else{
				m_Succeeded=false;
			}
			
			// Moved to adminConfig
			//adminNeighbor_command.execCommand(connection, conf);
			//if(adminNeighbor_command.succeeded()){
			//	m_AdminReplicate.append(adminNeighbor_command.getAdminNeighbor());
			//}else{
			//	m_Succeeded=false;
			//}
			
			adminLocaleSchema_command.execCommand(connection, conf);
			if(adminLocaleSchema_command.succeeded()){
				m_AdminReplicate.append(adminLocaleSchema_command.getLocaleSchema());
			}else{
				m_Succeeded=false;
			}
			
			adminProfileSchema_command.execCommand(connection, conf);
			if(adminProfileSchema_command.succeeded()){
				m_AdminReplicate.append(adminProfileSchema_command.getProfileSchema());
			}else{
				m_Succeeded=false;
			}
			
			adminProfile_command.execCommand(connection, conf);
			if(adminProfile_command.succeeded()){
				m_AdminReplicate.append(adminProfile_command.getCollection());
			}else{
				m_Succeeded=false;
			}
			
		}
		catch(Exception e){
			m_Succeeded=false;
			throw e; 
		}

	}

	public String getCommandName() {
		return "DumpAdminCommand";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
