package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.expr.AbstractData;
import org.livee.data.expr.DataFactory;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class GetAdminLocaleSchemaCommand implements IDbCommand {

	private boolean m_Succeeded;
	private AbstractData m_Locales;
	
	protected GetAdminLocaleSchemaCommand(){
		m_Succeeded=false;
	}
	
	public boolean succeeded(){
		return m_Succeeded;
	}
	
	public AbstractData getLocaleSchema(){
		return m_Locales;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
	
		DataFactory df=DataFactory.getInstance();
		m_Locales=df.create("LocaleCollection");
		
		java.sql.Statement stmt=null;

		try{
			String sql;
			stmt=connection.createStatement();
						
			sql="SELECT name,language_code FROM admin.locale ORDER BY name";
			java.sql.ResultSet rset=stmt.executeQuery(sql);
			while(rset.next()){
				String name=rset.getString(1);
				String code=rset.getString(2);
				AbstractData locale=df.create("Locale");
				locale.setAttribute("name", name);
				locale.setAttribute("code", code);
				
				m_Locales.append(locale);
			}
			rset.close();
			m_Succeeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}

	}

	public String getCommandName() {
		return "GetAdminLocaleSchemaCommand";
	}

	public short getImplVersion() {
		return 0x0100;
	}
}
