package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.expr.AbstractData;
import org.livee.data.expr.DataFactory;
import org.json.*;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class GetAdminNeighborCommand implements IDbCommand {

	private boolean m_Succeeded;
	private AbstractData m_AdminNeighbor;
	
	public GetAdminNeighborCommand(){
		m_Succeeded=false;
	}
	
	public boolean succeeded(){
		return m_Succeeded;
	}
	
	public AbstractData getAdminNeighbor(){
		return m_AdminNeighbor;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
	
		DataFactory df=DataFactory.getInstance();
		m_AdminNeighbor=df.create("adminNeighbor");
		
		java.sql.Statement stmt=null;

		try{
			String sql;
			stmt=connection.createStatement();
						
			sql="SELECT name,type,server FROM admin.neighbor ORDER BY name";
			java.sql.ResultSet rset=stmt.executeQuery(sql);
			while(rset.next()){
				String name=rset.getString(1);
				String type=rset.getString(2);
				String server=rset.getString(3);
				AbstractData adminSite=df.create("adminSite");
				adminSite.setAttribute("name", name);
				adminSite.setAttribute("type", type);
				
				try{
					JSONArray json_server=new JSONArray(server);
					int length=json_server.length();
					for(int i=0;i<length;i++){
						JSONObject map=json_server.getJSONObject(i);
						AbstractData adminServer=df.create("adminServer");

						if(!map.isNull("service")){
							String service=map.getString("service");
							adminServer.setAttribute("service", service);
						}
						if(!map.isNull("url")){
							String url=map.getString("url");
							adminServer.setAttribute("url",url);
						}
						if(!map.isNull("linkStatus")){
							String linkStatus=map.getString("linkStatus");
							adminServer.setAttribute("linkStatus", linkStatus);
						}
						adminSite.append(adminServer);
					}
				}catch(Exception e){e.printStackTrace();};
				
				m_AdminNeighbor.append(adminSite);
			}
			rset.close();
			m_Succeeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}

	}

	public String getCommandName() {
		return "GetAdminNeighborCommand";
	}

	public short getImplVersion() {
		return 0x0100;
	}
}
