package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.exception.LiveEException;
import org.livee.data.exception.SensorNotFoundLiveEException;
import org.livee.data.expr.AbstractData;
import org.livee.data.expr.DataFactory;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class GetAggMonthCommand implements IDbCommand {


	boolean m_bSucceeded;
	
	String m_ID;
	String m_StartTimestamp;
	String m_EndTimestamp;
	java.util.TimeZone m_TimeZone;
	String m_Locale;
	
	AbstractData m_CombinedSensor;
	
	public boolean succeeded(){
		return m_bSucceeded;
	}
	
	public AbstractData getCombinedSensor(){
		return m_CombinedSensor;
	}
	
	
	public GetAggMonthCommand(String id,String start, String end, java.util.TimeZone tz,String locale){
		m_ID=id;
		m_StartTimestamp=start;
		m_EndTimestamp=end;
		m_TimeZone=tz;
		m_Locale=locale;
				
		m_bSucceeded=false;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {
		
		DataFactory df=DataFactory.getInstance();
		m_CombinedSensor=df.create("combined");
		String language_code=null;
		String runtime_language_code=null;
		java.sql.Statement stmt=null;

		
		try{
			String sql;
			stmt=connection.createStatement();
						
			sql="SELECT language_code FROM admin.locale WHERE name=\'"+m_Locale+"\'";
			java.sql.ResultSet rset=stmt.executeQuery(sql);
			if(rset.next()){
				language_code=rset.getString(1);
				runtime_language_code="_"+language_code;
			}else{
				language_code="eng";
				runtime_language_code="_eng";
			}
						
			sql="SELECT sensor.combined.id, created, latitude, longitude, attrs FROM sensor.combined LEFT JOIN profile.combined ON sensor.combined.id=profile.combined.id WHERE sensor.combined.id=\'"+m_ID+"\'";
			rset=stmt.executeQuery(sql);
			if(rset.next()){
				String id=rset.getString("id");
				m_CombinedSensor.setAttribute("id", id);
				m_CombinedSensor.setAttribute("latitude", rset.getString("latitude"));
				m_CombinedSensor.setAttribute("longitude", rset.getString("longitude"));
				String attrs=rset.getString("attrs");
				try{
					java.util.Map map=new java.util.Hashtable();
					org.json.JSONObject jobj=new org.json.JSONObject(attrs);
					java.util.Iterator itr=jobj.keys();
					while(itr.hasNext()){
						String key=(String)itr.next();
						
						int index=key.lastIndexOf("_");
						if(language_code!=null && index>0){
							String generic_key=key.substring(0,index);
							if(key.endsWith(runtime_language_code)){
								map.put(generic_key, key);
							}else if(key.endsWith("_eng")){
								if(!map.containsKey(generic_key)){
									map.put(generic_key, key);
								}
							}
						}//else if(!map.containsKey(key)){  // Commented out 2009-01-18 by Hideya Ochiai
							map.put(key, key);
						//}
					}

					itr=map.keySet().iterator();
					while(itr.hasNext()){
						String generic_key=(String)itr.next();
						String key=(String)map.get(generic_key);
						m_CombinedSensor.setAttribute(generic_key, jobj.getString(key));
					}

				}
				catch(Exception e){
					e.printStackTrace();
				}
			}else{
				// Not Found
				m_CombinedSensor=null;
				throw new SensorNotFoundLiveEException(m_ID);
			}
			rset.close();
						
			// Element Archive Reader
			java.util.Map m_ElementMap=new java.util.Hashtable();
			sql="SELECT sensor.element.id,sensorType,attrs FROM sensor.element LEFT JOIN profile.element ON sensor.element.id=profile.element.id WHERE sensor.element.combined_id=\'"+m_ID+"\'";
			rset=stmt.executeQuery(sql);
			while(rset.next()){
				AbstractData element=df.create("element");
				m_CombinedSensor.append(element);
				
				// Set "element" Done on 2009-01-18 by Hideya Ochiai
				String id=rset.getString("id");
				String sensorType=rset.getString("sensorType");
				String attrs=rset.getString("attrs");
				
				element.setAttribute("id", id);
				if(sensorType!=null){
					element.setAttribute("sensorType",sensorType);
				}
				try{
					org.json.JSONObject jobj=new org.json.JSONObject(attrs);
					java.util.Iterator itr=jobj.keys();
					while(itr.hasNext()){
						String key=(String)itr.next();
						element.setAttribute(key, jobj.getString(key));
					}
				}
				catch(Exception e){
				//	e.printStackTrace();
				}

				m_ElementMap.put(id, element);
			}
			rset.close();
						
			// Data Archive Reader
			if(m_EndTimestamp==null){
				sql="SELECT " +
						"aggregation.month.id, " +
						"aggregation.month.aggregationType, " +
						"aggregation.month.time, " +
						"aggregation.month.attrs, " +
						"aggregation.month.value, " +
						"aggregation.month.sValue " +
					"FROM sensor.element, aggregation.month WHERE sensor.element.id=aggregation.month.id " +
					"AND sensor.element.combined_id=\'"+m_ID+"\' AND aggregation.month.time>=\'"+m_StartTimestamp+"\' ORDER BY aggregation.month.id, aggregation.month.time";
			}else{
				sql="SELECT " +
					"aggregation.month.id, " +
					"aggregation.month.aggregationType, " +
					"aggregation.month.time, " +
					"aggregation.month.attrs, " +
					"aggregation.month.value, " +
					"aggregation.month.sValue " +
				"FROM sensor.element, aggregation.month WHERE sensor.element.id=aggregation.month.id " +
				"AND sensor.element.combined_id=\'"+m_ID+"\' AND aggregation.month.time>=\'"+m_StartTimestamp+"\' AND aggregation.month.time<=\'"+m_EndTimestamp+"\' ORDER BY aggregation.month.id, aggregation.month.time";
			}
			rset=stmt.executeQuery(sql);
			while(rset.next()){
				AbstractData value=df.create("aggValue");
				String id=rset.getString("id");
				
				// Set Attributes of "aggValue"
				String time=null;
				try{
					time=org.livee.util.W3CTimestamp.toString(rset.getTimestamp("time"),m_TimeZone);
				}
				catch(Exception e){
					time=rset.getString("time");
				}
				if(time!=null){
					value.setAttribute("time",time);
				}
				String aggType=rset.getString("aggregationType");
				if(aggType!=null && !aggType.equals("")){
					value.setAttribute("aggType", aggType);
				}
				value.setAttribute("timeScope", "month");

				String attrs=rset.getString("attrs");
				try{
					org.json.JSONObject jobj=new org.json.JSONObject(attrs);
					java.util.Iterator itr=jobj.keys();
					while(itr.hasNext()){
						String key=(String)itr.next();
						value.setAttribute(key, jobj.getString(key));
					}
				}
				catch(Exception exp){
					exp.printStackTrace();
				}
				
				String text=rset.getString("value");
				if(text==null || text.equals("")){
					text=rset.getString("sValue");
				}
				if(text!=null){
					value.setTextContent(text);
				}
				
				Object obj=m_ElementMap.get(id);
				if(obj!=null){
					AbstractData element=(AbstractData)obj;
					element.append(value);
				}
			}
			rset.close();

			m_bSucceeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}

	}

	public String getCommandName() {
		return "GetAggMonth";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
