package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.exception.LiveEException;
import org.livee.data.expr.AbstractData;


/**
 * @author Hideya Ochiai, Akihiro Sugiyama, The University of Tokyo
 * Created: before 2008-01-01
 */
public class GetProfileAllCommand implements IDbCommand {

	/**
	 * Response
	 */

	AbstractData m_Collection = null;

	public GetProfileAllCommand() {
		m_Succeeded = false;
	}

	boolean m_Succeeded;

	public boolean succeeded() {
		return m_Succeeded;
	}

	public AbstractData getResult() {
		return m_Collection;
	}

	public String getCommandName() {
		return "GetProfileAll";
	}

	public short getImplVersion() {
		return 0x0100;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {
		
		java.sql.Statement stmt=null;
		org.livee.data.expr.DataFactory df = org.livee.data.expr.DataFactory.getInstance();
		m_Collection=df.create("collection");
		
		java.util.Map combinedMap=new java.util.Hashtable();
		
		try {
			stmt = connection.createStatement();
			String sql = "SELECT sensor.combined.id,created,latitude,longitude,attrs FROM sensor.combined LEFT JOIN profile.combined ON sensor.combined.id=profile.combined.id WHERE sensor.combined.working=\'t\' ORDER BY sensor.combined.id;";
			java.sql.ResultSet rset = stmt.executeQuery(sql);
			while(rset.next()){
				AbstractData combined = df.create("combined");
				m_Collection.append(combined);
				
				String id=rset.getString("id");
				String latitude=rset.getString("latitude");
				String longitude=rset.getString("longitude");
				
				combinedMap.put(id, combined);
				
				combined.setAttribute("id",id);
				combined.setAttribute("latitude", latitude);
				combined.setAttribute("longitude",longitude);
				
				try {
					org.json.JSONObject jo = new org.json.JSONObject(rset.getString("attrs"));
					java.util.Iterator itr = jo.keys();
					while (itr.hasNext()) {
						String key = (String) itr.next();
						combined.setAttribute(key, jo.getString(key));
					}
				} catch (Exception e) {
				//	e.printStackTrace();
				}
				
			}			
			rset.close();

			
			
			sql = "SELECT profile.element.id, sensor.element.combined_id, profile.element.created, profile.element.sensorType, profile.element.attrs FROM sensor.element RIGHT JOIN profile.element ON sensor.element.id=profile.element.id;";
			rset = stmt.executeQuery(sql);
			while (rset.next()) {
				AbstractData element = df.create("element");
				String id=rset.getString("id");
				String combined_id=rset.getString("combined_id");
				String sensorType=rset.getString("sensorType");
				AbstractData combined=(AbstractData)combinedMap.get(combined_id);
				
				if(combined!=null){
					combined.append(element);
					element.setAttribute("id", id);
					element.setAttribute("sensorType",sensorType);
					
					String attrs = rset.getString("attrs");
					try {
						org.json.JSONObject jo = new org.json.JSONObject(attrs);
						java.util.Iterator itr = jo.keys();
						while (itr.hasNext()) {
							String key = (String) itr.next();
							element.setAttribute(key, jo.getString(key));
						}
					} catch (Exception e) {
						// e.printStackTrace();
					}
					
				}
			}
			m_Succeeded = true;
		}		
		catch (Exception e) {
			throw e;
		}
		finally{
			stmt.close();
		}
	}

}
