package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.expr.AbstractData;

/**
 * GetProfileSchemaCommand is obsolete
 * @author Hideya Ochiai, The University of Tokyo
 * @see GetAdminProfileSchemaCommand
 * Created: before 2008-01-01
 */
public class GetProfileSchemaCommand implements IDbCommand {

	private boolean m_Succeeded;
	private AbstractData m_ProfileSchema;
	
	protected GetProfileSchemaCommand(){
		m_ProfileSchema=null;
		m_Succeeded=false;
	}
	
	public boolean succeeded(){
		return m_Succeeded;
	}
	
	public AbstractData getProfileSchema(){
		return m_ProfileSchema;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {

		java.sql.Statement stmt = null;

		try{
			org.livee.data.expr.DataFactory df=org.livee.data.expr.DataFactory.getInstance();
			org.livee.data.expr.AbstractData profileSchema=df.create("profileSchema");
			
			stmt=connection.createStatement();
			java.sql.ResultSet rset=stmt.executeQuery("SELECT * FROM admin.schema ORDER BY name;");
			
			while(rset.next()){
				org.livee.data.expr.AbstractData schema=df.create("schema");
				schema.setAttribute("name", rset.getString("name"));
				schema.setAttribute("type", rset.getString("type"));
				schema.setAttribute("class", rset.getString("class"));
				schema.setAttribute("value", rset.getString("value"));
				schema.setAttribute("description", rset.getString("description"));
				schema.setAttribute("visibility", rset.getString("visibility"));
				if(rset.getBoolean("multilanguage")){
					schema.setAttribute("multilanguage", "true");
				}
				profileSchema.append(schema);
			}
			rset.close();
			
			m_ProfileSchema=profileSchema;
			m_Succeeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}
	}

	public String getCommandName() {
		return "GetProfileSchemaCommand";
	}

	public short getImplVersion() {
		return 0x0100;
	}


}
