package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.expr.AbstractData;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class SetAdminConfigCommand implements IDbCommand {

	private boolean m_Succeeded;
	private AbstractData m_AdminConfig;
	
	protected SetAdminConfigCommand(AbstractData adminConfig){
		m_AdminConfig=adminConfig;
		m_Succeeded=false;
	}
	
	public boolean succeeded(){
		return m_Succeeded;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {
		
		String sql;
		java.sql.Statement stmt=null;

		try{
			stmt = connection.createStatement();
			
			stmt.execute("START TRANSACTION;");
			stmt.execute("DELETE FROM admin.conf;");

			java.util.Iterator itr=m_AdminConfig.getChildren().iterator();
			while(itr.hasNext()){
				AbstractData data=(AbstractData)itr.next();
				if(data.getDataType().equals("AdminConfigParameter")){
					AbstractData param=data;
					String name=param.getAttribute("name");
					String value=param.getAttribute("value");
				
					if(name==null){
						continue;
					}
					StringBuilder sb=new StringBuilder("INSERT INTO admin.conf (name,value) VALUES (\'");
					sb.append(name);
					sb.append("\',");
					if(value!=null){
						sb.append("\'");
						sb.append(value);
						sb.append("\');");
					}else{
						sb.append("null);");
					}
					sql=sb.toString();
					
					//	System.out.println(sql);
				
					stmt.execute(sql);
				}else if(data.getDataType().equals("AdminNeighbor")){
					SetAdminNeighborCommand command=new SetAdminNeighborCommand(data);
					command.execCommand(connection, conf);
				}
			}
			stmt.execute("COMMIT;");
			m_Succeeded=true;
		}
		catch(Exception e){
			stmt.execute("ROLLBACK;");
			throw e;
		}
		finally{
			stmt.close();
		}
	}

	public String getCommandName() {
		return "SetAdminConfigCommand";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
