package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.expr.AbstractData;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class SetAdminLocaleSchemaCommand implements IDbCommand {

	private boolean m_Succeeded;
	private AbstractData m_LocaleCollection;
	
	protected SetAdminLocaleSchemaCommand(AbstractData localeCollection){
		m_LocaleCollection=localeCollection;
		m_Succeeded=false;
	}
	
	public boolean succeeded(){
		return m_Succeeded;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {
		
		String sql;
		java.sql.Statement stmt=null;

		try{
			stmt = connection.createStatement();
			stmt.execute("START TRANSACTION;");
			stmt.execute("DELETE FROM admin.locale;");

			java.util.Iterator itr=m_LocaleCollection.getChildren().iterator();
			while(itr.hasNext()){
				AbstractData locale=(AbstractData)itr.next();
				String name=locale.getAttribute("name");
				String code=locale.getAttribute("code");
				
				if(name==null || code==null){
					continue;
				}
				
				StringBuilder sb=new StringBuilder("INSERT INTO admin.locale (name,language_code) VALUES (\'");
				
				sb.append(name);
				sb.append("\',\'");
				sb.append(code);
				sb.append("\');");

				sql=sb.toString();
				
				//System.out.println(sql);
				
				stmt.execute(sql);
			}
			
			stmt.execute("COMMIT;");
			
			m_Succeeded=true;
		}
		catch(Exception e){
			stmt.execute("ROLLBACK;");
			throw e;
		}
		finally{
			stmt.close();
		}
	}

	public String getCommandName() {
		return "SetAdminLocaleSchema";
	}

	public short getImplVersion() {
		return 0x0100;
	}


}
