package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Iterator;
import java.util.Map;

import org.json.JSONObject;
import org.livee.data.expr.AbstractData;
import org.livee.data.tools.ProfileSchemaProcessor;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class SetAdminProfileReplicateCommand implements IDbCommand {

	/**
	 * Request
	 */
	AbstractData m_Collection = null;

	/**
	 * Response
	 */
	boolean m_Succeeded;

	private SetAdminProfileReplicateCommand() {
	}

	protected SetAdminProfileReplicateCommand(AbstractData collection) {
		m_Collection = collection;
		m_Succeeded = false;
	}

	public String getCommandName() {
		return "SetAdminProfileReplicate";
	}

	public short getImplVersion() {
		return 0x0100;
	}

	public boolean succeeded() {
		return m_Succeeded;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {

		java.sql.Statement stmt = null;
		ProfileSchemaProcessor schema=ProfileSchemaProcessor.getInstance();
		
		try {
			
			stmt = connection.createStatement();
			
			stmt.execute("START TRANSACTION;");
			stmt.execute("DELETE FROM admin.combined;");
			String sql=null;
			
			Iterator itr=m_Collection.getChildren().iterator();
			while(itr.hasNext()){
				AbstractData combined=(AbstractData)itr.next();
				String combined_id = combined.getAttribute("id");
				
				if(combined_id==null){
					continue;
				}
				
				String site=null;
				String updated=null;
				
				// Attribute Iteration
				Map map=combined.getAttributes();
				site=(String)map.get("adminSite");
				updated=(String)map.get("adminUpdated");
				map.remove("adminSite");
				map.remove("adminUpdated");
				map.remove("class");
				map.remove("xmlns");
				map.remove("id");

				if(site==null || updated==null){
					System.err.println("Warning: "+combined_id+" not successfully replicated..");
					continue;
				}
				
				sql="INSERT INTO admin.combined (id,site,updated) VALUES (\'"+combined_id+"\',\'"+site+"\',\'"+updated+"\');";
				//System.out.println(sql);
				stmt.execute(sql);
				
				Iterator attr_itr=map.keySet().iterator();
				while(attr_itr.hasNext()){
					String key = (String) attr_itr.next();
					String value = (String) map.get(key);

					sql=null;
					int type=schema.getDataType(key);
					switch(type){
					case ProfileSchemaProcessor.Schema.SCHEMA_TYPE_BOOLEAN:
						if(value!=null && value.startsWith("t")){
							sql="INSERT INTO admin.combined_index (id,name,sValue) VALUES (\'"+combined_id+"\',\'"+key+"\',\'true\');";
						}else{
							sql="INSERT INTO admin.combined_index (id,name,sValue) VALUES (\'"+combined_id+"\',\'"+key+"\',\'false\');";
						}
						break;
					case ProfileSchemaProcessor.Schema.SCHEMA_TYPE_FLOAT:
						sql="INSERT INTO admin.combined_index (id,name,fValue) VALUES (\'"+combined_id+"\',\'"+key+"\',"+value+");";
						break;
					case ProfileSchemaProcessor.Schema.SCHEMA_TYPE_INTEGER:
						sql="INSERT INTO admin.combined_index (id,name,iValue) VALUES (\'"+combined_id+"\',\'"+key+"\',"+value+");";
						break;
					case ProfileSchemaProcessor.Schema.SCHEMA_TYPE_STRING:
					default:
						sql="INSERT INTO admin.combined_index (id,name,sValue) VALUES (\'"+combined_id+"\',\'"+key+"\',\'"+value+"\');";
						break;
					case ProfileSchemaProcessor.Schema.SCHEMA_TYPE_TIME:
						sql="INSERT INTO admin.combined_index (id,name,tValue) VALUES (\'"+combined_id+"\',\'"+key+"\',\'"+value+"\');";
						break;
					case ProfileSchemaProcessor.Schema.SCHEMA_TYPE_NULL:
					
						break;
					}
					if(sql!=null){
						stmt.execute(sql);
					}
	
				}				
				
				Iterator itr2=combined.getChildren().iterator();
				while(itr2.hasNext()){
					AbstractData element=(AbstractData)itr2.next();
					String element_id=element.getAttribute("id");
					
					sql="INSERT INTO admin.element (id,combined_id) VALUES (\'"+element_id+"\',\'"+combined_id+"\');";
					//System.out.println(sql);
					stmt.execute(sql);
					
					Map element_map=element.getAttributes();
					element_map.remove("id");
					attr_itr = element_map.keySet().iterator();
					while(attr_itr.hasNext()){
						String key = (String) attr_itr.next();
						String value = (String) element_map.get(key);

						sql=null;
						int type=schema.getDataType(key);
						switch(type){
						case ProfileSchemaProcessor.Schema.SCHEMA_TYPE_BOOLEAN:
							if(value!=null && value.startsWith("t")){
								sql="INSERT INTO admin.element_index (id,name,sValue) VALUES (\'"+element_id+"\',\'"+key+"\',\'true\');";
							}else{
								sql="INSERT INTO admin.element_index (id,name,sValue) VALUES (\'"+element_id+"\',\'"+key+"\',\'false\');";
							}
							break;
						case ProfileSchemaProcessor.Schema.SCHEMA_TYPE_FLOAT:
							sql="INSERT INTO admin.element_index (id,name,fValue) VALUES (\'"+element_id+"\',\'"+key+"\',"+value+");";
							break;
						case ProfileSchemaProcessor.Schema.SCHEMA_TYPE_INTEGER:
							sql="INSERT INTO admin.element_index (id,name,iValue) VALUES (\'"+element_id+"\',\'"+key+"\',"+value+");";
							break;
						case ProfileSchemaProcessor.Schema.SCHEMA_TYPE_STRING:
						default:
							sql="INSERT INTO admin.element_index (id,name,sValue) VALUES (\'"+element_id+"\',\'"+key+"\',\'"+value+"\');";
							break;
						case ProfileSchemaProcessor.Schema.SCHEMA_TYPE_TIME:
							sql="INSERT INTO admin.element_index (id,name,tValue) VALUES (\'"+element_id+"\',\'"+key+"\',\'"+value+"\');";
							break;
						case ProfileSchemaProcessor.Schema.SCHEMA_TYPE_NULL:
						
							break;
						}
						if(sql!=null){
							//System.out.println(sql);
							stmt.execute(sql);
						}
					}
					
				}
			}
			
			stmt.execute("COMMIT;");
			m_Succeeded=true;
		} catch (Exception e) {
			stmt.execute("ROLLBACK;");
			throw e;
		}
		finally{
			stmt.close();
		}
	}
	
}



