package org.livee.data.db;

/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Iterator;
import java.util.Map;

import java.util.*;

import org.livee.data.exception.AuthenticationLiveEException;
import org.livee.data.expr.AbstractData;
import org.json.*;
import org.w3c.dom.*;

/**
 * @author Hideya Ochiai, Akihiro Sugiyama, The University of Tokyo
 * Created: before 2008-01-01
 */
public class SetProfileCommand implements IDbCommand {

	/**
	 * Request
	 */
	AbstractData m_CombinedSensor = null;

	String m_passwd = null;

	/**
	 * Response
	 */
	boolean m_Succeeded;

	private SetProfileCommand() {
	}

	protected SetProfileCommand(AbstractData combinedSensor, String passwd) {
		m_CombinedSensor = combinedSensor;
		try {
			m_passwd = org.livee.util.Security.SHA1(passwd);
		} catch (Exception e) {
			e.printStackTrace();
		}
		m_Succeeded = false;
	}

	public String getCommandName() {
		return "SetProfileCommand";
	}

	public short getImplVersion() {
		return 0x0100;
	}

	public boolean succeeded() {
		return m_Succeeded;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {

		java.sql.Statement stmt = null;

		try {
			
			String id = null;
			String latitude = null;
			String longitude = null;
			String default_created = java.util.Calendar.getInstance().getTime().toString();
			String created = default_created;
			JSONObject attrs = new JSONObject();

			// Attribute Iteration
			Map map = m_CombinedSensor.getAttributes();
			Iterator itr = map.keySet().iterator();
			while (itr.hasNext()) {
				String key = (String) itr.next();
				String value = (String) map.get(key);

				if (key.equals("latitude")) {
					latitude = value;
				} else if (key.equals("longitude")) {
					longitude = value;
				} else if (key.equals("created")) {
					created = value;
				} else if (key.equals("id")) {
					id = value;
					m_CombinedId=id;
				} else {
					attrs.put(key, value);
				}
			}

			stmt = connection.createStatement();
			stmt.execute("START TRANSACTION;");

			String sql = "SELECT passwd from sensor.combined where id=\'"+id+"\' AND passwd=\'"+m_passwd+"\';";
			//System.out.println(sql);
			java.sql.ResultSet rset=stmt.executeQuery(sql);
			if(!rset.next()){
				rset.close();
				throw new AuthenticationLiveEException(id,m_passwd);
			}
			rset.close();			
			
			updateCombined(stmt, id, created, latitude, longitude, attrs.toString());

			// Element Iteration
			deleteElement(stmt, id, created);
			java.util.Collection elements = m_CombinedSensor.getChildren();
			java.util.Iterator element_itr = elements.iterator();
			while (element_itr.hasNext()) {
				AbstractData element = (AbstractData) element_itr.next();

				id = null;
				created = default_created;
				attrs = new org.json.JSONObject();
				String sensorType = null;

				map=element.getAttributes();
				itr = map.keySet().iterator();

				while (itr.hasNext()) {
					String key = (String) itr.next();
					String value = (String) map.get(key);

					if (key.equals("id")) {
						id = value;
					} else if (key.equals("created")) {
						created = value;
					} else if (key.equals("sensorType")) {
						sensorType = value;
					} else {
						attrs.put(key, value);
					}
				}
				updateElement(stmt, id, created, sensorType, attrs.toString());
			}
			stmt.execute("COMMIT;");
			m_Succeeded=true;
			
		} catch (Exception e) {
			stmt.execute("ROLLBACK;");
			throw e;
		}
		finally{
			stmt.close();
		}
	}

	private void updateCombined(java.sql.Statement stmt, String id,
			String created, String latitude, String longitude, String attrs)
			throws Exception {

		String sql = "UPDATE profile.combined_archive SET expired=\'" + created
				+ "\' WHERE id=\'" + id + "\' and expired IS NULL;";

		// debug
		//System.out.println(sql);
		stmt.executeUpdate(sql);

		StringBuilder sb=new StringBuilder("INSERT INTO profile.combined_archive (id, created, latitude,longitude,attrs) VALUES (\'");
		sb.append(id);
		sb.append("\',\'");
		sb.append(created);
		sb.append("\',");
		sb.append(latitude);
		sb.append(",");
		sb.append(longitude);
		sb.append(",\'");
		sb.append(attrs.replaceAll("\'","\\\\\'"));
		sb.append("\');");
		
		sql = sb.toString();
		
		// debug
		//System.out.println(sql);

		stmt.execute(sql);

	}

	private String m_CombinedId;
	private void updateElement(java.sql.Statement stmt, String id,
			String created, String sensorType, String attrs) throws Exception {

		String sql = "UPDATE sensor.element SET working=true WHERE id=\'" + id + "\';";
		// debug
		//System.out.println(sql);
		if(stmt.executeUpdate(sql)==0){
			sql = "INSERT INTO sensor.element (id, combined_id, working) VALUES (\'"+id+"\', \'"+m_CombinedId+"\', true);";
			stmt.execute(sql);
		}

		StringBuilder sb=new StringBuilder("INSERT INTO profile.element_archive (id, created, expired, sensorType, attrs) VALUES (\'");
		sb.append(id);
		sb.append("\',\'");
		sb.append(created);
		sb.append("\',null,");
		if(sensorType!=null){ // Modified so as not to be appeared sensorType="null" 
			sb.append("\'");  // 2009-01-18 by Hideya Ochiai
			sb.append(sensorType);
			sb.append("\',");
		}else{
			sb.append("null,");
		}
		sb.append("\'");
		sb.append(attrs.replaceAll("\'", "\\\\\'"));
		sb.append("\');");
		
		sql = sb.toString();

		// debug
		//System.out.println(sql);

		stmt.execute(sql);

	}

	private void deleteElement(java.sql.Statement stmt, String id, String created) throws Exception{
		String sql = "UPDATE sensor.element SET working=false WHERE combined_id=\'"
				+ id + "\';";
		//System.out.println(sql);
		stmt.executeUpdate(sql);

		sql = "UPDATE profile.element_archive SET expired=\'"+created+"\' WHERE id IN (SELECT id FROM sensor.element WHERE combined_id=\'"
				+ id + "\') AND expired IS NULL;";
		stmt.executeUpdate(sql);
	}

}
