package org.livee.data.db;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.expr.*;
import org.livee.util.Xml;
import org.w3c.dom.*;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class TestDbCommand {

	public static void main(String[] args) throws Exception{
		for(int i=0;i<10000;i++){
			testAggregation();
		}
	}

	public static void testAggregation() throws Exception {
		
		DatabaseAdapter db=DatabaseAdapter.getInstance();
		AggregationCommand command=new AggregationCommand();
		
		db.postCommand(command);
		
		if(command.succeeded()){
			System.out.println("Succeeded");
		}
	}
	
	public static void testGetAggHour() throws Exception {
		
		Document doc=Xml.newDocument();
		DatabaseAdapter db=DatabaseAdapter.getInstance();
		GetAggHourCommand command=new GetAggHourCommand("live-e.org/Virtual/00000000/","2007-06-01T00:00:00+09:00",null,java.util.TimeZone.getDefault(),"Japanese");
		
		db.postCommand(command);
		
		if(command.succeeded()){
			AbstractData data=command.getCombinedSensor();
			System.out.println(Xml.toString(data.getXml(doc)));
		}
	}
	
	public static void testGetAdminNeighbor() throws Exception {
		
		Document doc=Xml.newDocument();
		DatabaseAdapter db=DatabaseAdapter.getInstance();
		GetAdminNeighborCommand command=CommandFactory.createGetAdminNeighborCommand();
		
		db.postCommand(command);
		
		if(command.succeeded()){
			AbstractData neighbor=command.getAdminNeighbor();
			System.out.println(Xml.toString(neighbor.getXml(doc)));
		}
		
	}
	
	public static void testSetAdminNeighbor() throws Exception {		
		
		Document doc=Xml.parseXmlFile("test/neighbor.xml");
		Element xml=doc.getDocumentElement();
		
		ParserFactory pf=ParserFactory.getInstance();
		AbstractParser neighbor_parser=pf.get("adminNeighbor");
		AbstractData neighbor=neighbor_parser.parse(xml);
		
		System.out.println(Xml.toString(neighbor.getXml(doc)));
		
		DatabaseAdapter db=DatabaseAdapter.getInstance();
		SetAdminNeighborCommand command=CommandFactory.createSetAdminNeighborCommmand(neighbor);
		
		db.postCommand(command);
	}

	public static void testGetAdminProfile() throws Exception {
		
		Document doc=Xml.newDocument();
		DatabaseAdapter db=DatabaseAdapter.getInstance();
		GetAdminProfileCommand command=CommandFactory.createGetAdminProfileCommand();
		
		db.postCommand(command);
		
		if(command.succeeded()){
			AbstractData profile=command.getCollection();
			System.out.println(Xml.toString(profile.getXml(doc)));
		}
		
	}
	
	public static void testSetAdminProfile() throws Exception {		
		
		Document doc=Xml.parseXmlFile("latestDataAll.xml");
		Element xml=doc.getDocumentElement();
		
		ParserFactory pf=ParserFactory.getInstance();
		AbstractParser collection_parser=pf.get("collection");
		AbstractData collection=collection_parser.parse(xml);
		
		System.out.println(Xml.toString(collection.getXml(doc)));
		
		DatabaseAdapter db=DatabaseAdapter.getInstance();
		SetAdminProfileCommand command=CommandFactory.createSetAdminProfileCommand("thai.live-e.org",collection);
		
		db.postCommand(command);
	}

/*		DatabaseAdapter db=DatabaseAdapter.getInstance();
		
		java.util.Calendar s=java.util.Calendar.getInstance();
		java.util.Calendar e=java.util.Calendar.getInstance();
		s.set(2007,02,01,0,0,0);
		e.set(2007,02,03,0,0,0);
		
		//GetViewProfileByAreaRectCommand command=new GetViewProfileByAreaRectCommand(31,23,140.23,122.9,java.util.TimeZone.getTimeZone("UTC"),"English");
		//GetViewProfileAllCommand command=new GetViewProfileAllCommand(java.util.TimeZone.getTimeZone("UTC"),"English");
		//GetViewProfileCommand command=new GetViewProfileCommand("1",java.util.TimeZone.getTimeZone("UTC"),"English");
		//GetViewProfileArchiveCommand command=new GetViewProfileArchiveCommand("1",s,e,java.util.TimeZone.getTimeZone("JST"),"English");
		GetArchiveCombinedDataCommand command=new GetArchiveCombinedDataCommand("1",s,e,java.util.TimeZone.getTimeZone("JST"),"English");
		db.postCommand(command);
		
		
		if(command.succeeded()){
			org.livee.data.expr.AbstractData collection=command.getCombinedSensor();
			if(collection!=null){
				org.w3c.dom.Document doc=org.livee.util.Xml.newDocument();
				org.w3c.dom.Element xml=collection.getXml(doc);			
				System.out.println(org.livee.util.Xml.toString(xml));
			}
			
		}
*/

}
