package org.livee.data.db;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;
import org.livee.util.*;


/**
 * @author Hideya Ochiai, Akihiro Sugiyama, The University of Tokyo
 * Created: before 2008-01-01
 */
public class UpdateAuthorizedSensorCommand implements IDbCommand {


	/**
	 * Request
	 */
	String m_ID;
	String m_Passwd;
	String m_OwnerName;
	String m_OwnerMailAddress;
	
	/**
	 * Response
	 */
	boolean m_Succeeded;

	public boolean succeeded(){
		return m_Succeeded;
	}
	
	private UpdateAuthorizedSensorCommand(){}
	
	public UpdateAuthorizedSensorCommand(String id,String new_passwd,String owner_name,String owner_mail_address){
		m_ID=id;
		m_OwnerName=owner_name;
		m_OwnerMailAddress=owner_mail_address;
		if(new_passwd!=null){
			try{
				m_Passwd=Security.SHA1(new_passwd);
			}catch(Exception e){
				e.printStackTrace();
			}
		}else{
			m_Passwd=new_passwd;
		}
		m_Succeeded=false;
	}
		
	public String getCommandName() {
		return "UpdateAuthorizedSensorCommand";
	}

	public short getImplVersion() {
		return 0x0100;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
			java.sql.Statement stmt=null;
			try{
				stmt=connection.createStatement();
				StringBuilder sb=new StringBuilder("UPDATE sensor.combined SET ");
				if(m_Passwd!=null){
					sb.append("passwd=\'");
					sb.append(m_Passwd);
					sb.append("\', ");
				}
				if(m_OwnerName!=null){
					sb.append("ownerName=\'");
					sb.append(m_OwnerName.replaceAll("\'", "\\\\\'"));
					sb.append("\', ");
				}
				if(m_OwnerMailAddress!=null){
					sb.append("ownerMailAddress=\'");
					sb.append(m_OwnerMailAddress);
					sb.append("\', ");
				}
				if(m_Passwd!=null || m_OwnerName!=null || m_OwnerMailAddress!=null){
					sb.deleteCharAt(sb.length()-1);
					sb.deleteCharAt(sb.length()-1);
					sb.append(" WHERE id=\'" + m_ID + "\';");
					String sql=sb.toString();
					stmt.executeUpdate(sql);
				}
				m_Succeeded=true;
			}
			catch(Exception e){
				throw e;
			}
			finally{
				stmt.close();				
			}
	}

	
}
