package org.livee.data.expr;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class AdminConfigParser implements AbstractParser {

	public String getParserName() {
		return "AdminConfig";
	}

	public AbstractData parse(Element xml) throws Exception {
		if(xml.getNamespaceURI().equals("http://live-e.org/Administrator/2007/03/")){
			// For Version 200703			
			if(xml.getTagName().equals("adminConfig")){
				
				AbstractData adminConfig=DataFactory.getInstance().create("adminConfig");

				Node node=xml.getFirstChild();
				while(node!=null){
					if(node.getNodeType()==Node.ELEMENT_NODE){
						Element e0=(Element)node;
						String tagName=e0.getTagName();
						AbstractParser parser=null;
						if(tagName.equals("parameter")){
							parser=ParserFactory.getInstance().get("adminConfigParameter");
						}else if(tagName.equals("neighbor")){
							parser=ParserFactory.getInstance().get("adminNeighbor");
						}
						if(parser!=null){
							AbstractData data=parser.parse(e0);
							if(data!=null){
								adminConfig.append(data);
							}
						}
					}
					node=node.getNextSibling();
				}
				
				return adminConfig;
			}else{
				// Syntax Error
				return null;
			}
		}
		
		return null;
	}

}
