package org.livee.data.expr;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.util.Collection;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class AdminNeighborData implements AbstractData {
	java.util.Collection m_AdminSiteCol;
	
	protected AdminNeighborData(){
		m_AdminSiteCol=new java.util.ArrayList();
	}

	public void append(AbstractData child) {
		if(child==null){
			return ;
		}
		if(child.getDataType().equals("AdminSite")){
			m_AdminSiteCol.add(child);
		}
	}

	public String getAttribute(String name) {
		return null;
	}

	public Map getAttributes() {
		return null;
	}

	public Collection getChildren() {
		return m_AdminSiteCol;
	}

	public String getDataType() {
		return "AdminNeighbor";
	}

	public String getTextContent() {
		return null;
	}

	public Element getXml(Document doc) {
		
		String ns="http://live-e.org/Administrator/2007/03/";
		Element neighbor=doc.createElementNS(ns, "neighbor");
		
		java.util.Iterator itr=m_AdminSiteCol.iterator();
		while(itr.hasNext()){
			Object obj=itr.next();
			if(obj!=null){
				AbstractData lc=(AbstractData)obj;
				neighbor.appendChild(lc.getXml(doc));
			}
		}
		
		return neighbor;
	}

	public void remove(AbstractData child) {}

	public void removeAttribute(String name) {}

	public void setAttribute(String name, String value) {}

	public void setTextContent(String text) {}
	
	
	public static final int NEIGHBOR_NONE=0;
	public static final int NEIGHBOR_PARENT=1;
	public static final int NEIGHBOR_CHILD=2;
	public static final int NEIGHBOR_COLLEAGUE=3;
	
	public static int getRelation(AbstractData base,AbstractData target){
		
		String bParent=null;
		String bColleague=null;
		java.util.ArrayList bChild=new java.util.ArrayList();
		String tParent=null;
		String tColleague=null;
		java.util.ArrayList tChild=new java.util.ArrayList();
		
		java.util.Iterator itr;
		itr=base.getChildren().iterator();
		while(itr.hasNext()){
			AbstractData site=(AbstractData)itr.next();
			String name=site.getAttribute("name");
			String type=site.getAttribute("type");
			if(type.equals("parent")){
				bParent=name;
			}else if(type.equals("colleague")){
				bColleague=name;
			}else if(type.equals("child")){
				bChild.add(name);
			}
		}
		
		itr=target.getChildren().iterator();
		while(itr.hasNext()){
			AbstractData site=(AbstractData)itr.next();
			String name=site.getAttribute("name");
			String type=site.getAttribute("type");
			if(type.equals("parent")){
				tParent=name;
			}else if(type.equals("colleague")){
				tColleague=name;
			}else if(type.equals("child")){
				tChild.add(name);
			}
		}
		

		if(bColleague!=null && bColleague.equals(tColleague)){
			return NEIGHBOR_COLLEAGUE;
		}
		
		if(bParent!=null && bColleague!=null){
			int size=tChild.size();
			if(bParent.equals(tColleague)){
				for(int i=0;i<size;i++){
					if(bColleague.equals(tChild.get(i))){
						return NEIGHBOR_PARENT;
					}
				}
			}
		}

		if(tParent!=null && tColleague!=null){
			int size=bChild.size();
			if(tParent.equals(bColleague)){
				for(int i=0;i<size;i++){
					if(tColleague.equals(bChild.get(i))){
						return NEIGHBOR_CHILD;
					}
				}
			}
		}

		
		return NEIGHBOR_NONE;
	}
}