package org.livee.data.expr;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class ElementSensorParser implements AbstractParser {

	public String getParserName() {
		return "ElementSensor";
	}

	public AbstractData parse(Element xml) throws Exception {
		
		String namespaceURI=xml.getNamespaceURI();
		if(namespaceURI!=null && namespaceURI.equals("http://live-e.org/DataType/2007/03/")){
			// For Version 200703			
			if(xml.getTagName().equals("sensor")){
				
				AbstractData element=DataFactory.getInstance().create("element");
				NamedNodeMap map=xml.getAttributes();
				int i;
				int length=map.getLength();
				for(i=0;i<length;i++){
					Node n=map.item(i);
					String name=n.getNodeName();
					String value=n.getNodeValue();
					element.setAttribute(name, value);
				}
				element.removeAttribute("xmlns");

				Node node=xml.getFirstChild();
				while(node!=null){
					if(node.getNodeType()==Node.ELEMENT_NODE){
						Element e0=(Element)node;
						String tagName=e0.getTagName();
						if(tagName.equals("value")){
							AbstractParser value_parser=ParserFactory.getInstance().get("value");
							AbstractData value=value_parser.parse(e0);
							if(value!=null){
								element.append(value);
							}
						}else if(tagName.equals("sValue")){
							AbstractParser svalue_parser=ParserFactory.getInstance().get("sValue");
							AbstractData svalue=svalue_parser.parse(e0);
							if(svalue!=null){
								element.append(svalue);
							}							
						}else if(tagName.equals("aggValue")){
							AbstractParser aggvalue_parser=ParserFactory.getInstance().get("aggValue");
							AbstractData aggvalue=aggvalue_parser.parse(e0);
							if(aggvalue!=null){
								element.append(aggvalue);
							}
						}
					}
					node=node.getNextSibling();
				}
				
				return element;
			}else{
				// Syntax Error
				return null;
			}

		}else{
			// For Version 200608
			if(xml.getTagName().equals("sensor")){
				
				AbstractData element=DataFactory.getInstance().create("element");
				NamedNodeMap map=xml.getAttributes();
				int i;
				int length=map.getLength();
				for(i=0;i<length;i++){
					Node n=map.item(i);
					String name=n.getNodeName();
					String value=n.getNodeValue();
					// Schema Translation
					if(name.equals("location")){
						name="organization";
					}
					element.setAttribute(name, value);
				}
				
				Node node=xml.getFirstChild();
				while(node!=null){
					if(node.getNodeType()==Node.ELEMENT_NODE){
						Element e0=(Element)node;
						String tagName=e0.getTagName();
						if(tagName.equals("value")){
							AbstractParser value_parser=ParserFactory.getInstance().get("value");
							AbstractData value=value_parser.parse(e0);
							if(value!=null){
								element.append(value);
							}
						}else if(tagName.equals("sValue")){
							AbstractParser svalue_parser=ParserFactory.getInstance().get("sValue");
							AbstractData svalue=svalue_parser.parse(e0);
							if(svalue!=null){
								element.append(svalue);
							}							
						}else if(tagName.equals("aggValue")){
							AbstractParser aggvalue_parser=ParserFactory.getInstance().get("aggValue");
							AbstractData aggvalue=aggvalue_parser.parse(e0);
							if(aggvalue!=null){
								element.append(aggvalue);
							}
						}
					}
					node=node.getNextSibling();
				}
				
				return element;
			}else{
				// Syntax Error
				return null;
			}
		}
		
		//return null;
	}

}
