package org.livee.data.expr;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class SchemaParser implements AbstractParser {

	public String getParserName() {
		return "Schema";
	}

	public AbstractData parse(Element xml) throws Exception {
		
		if(xml.getNamespaceURI().equals("http://live-e.org/Schema/2007/03/")){
			// For Version 200703			
			if(xml.getTagName().equals("schema")){
				
				AbstractData schema=DataFactory.getInstance().create("schema");

				String name=xml.getAttribute("name");
				String type=xml.getAttribute("type");
				String value=xml.getAttribute("value");
				String multilanguage=xml.getAttribute("multilanguage");
				String description=xml.getAttribute("description");
				String visibility=xml.getAttribute("visibility");
				String clazz=xml.getAttribute("class");
				String delegate=xml.getAttribute("delegate");
				
				if(name!=null && !name.equals("")){
					schema.setAttribute("name", name);
				}				
				if(type!=null && !type.equals("")){
					schema.setAttribute("type", type);
				}
				if(value!=null && !value.equals("")){
					schema.setAttribute("value", value);
				}
				if(clazz!=null && !clazz.equals("")){
					schema.setAttribute("class", clazz);
				}
				if(multilanguage!=null && multilanguage.equals("true")){
					schema.setAttribute("multilanguage","true");
				}
				if(description!=null && !description.equals("")){
					schema.setAttribute("description", description);
				}
				if(visibility!=null && !visibility.equals("")){
					schema.setAttribute("visibility", visibility);
				}
				if(delegate!=null && !delegate.equals("")){
					schema.setAttribute("delegate", delegate);
				}
				
				return schema;
			}else{
				// Syntax Error
				return null;
			}
		}
		
		return null;
	}

}
