package org.livee.data.query;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.expr.AbstractData;
import org.livee.data.expr.AbstractParser;
import org.livee.data.query.DataFactory;
import org.w3c.dom.Element;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class FilterParser implements AbstractParser {

	public String getParserName() {
		return "Filter";
	}

	public AbstractData parse(Element xml) throws Exception {
		
		if(xml.getNamespaceURI().equals("http://live-e.org/Query/2007/03/")){
			// For Version 200703			
			if(xml.getTagName().equals("filter")){
				
				AbstractData filter=DataFactory.getInstance().create("filter");

				String targetClass=xml.getAttribute("targetClass");
				String attrName=xml.getAttribute("attrName");
				String eq=xml.getAttribute("eq");
				String neq=xml.getAttribute("neq");
				String lt=xml.getAttribute("lt");
				String gt=xml.getAttribute("gt");
				String lteq=xml.getAttribute("lteq");
				String gteq=xml.getAttribute("gteq");
				String contains=xml.getAttribute("contains");
				
				if(targetClass!=null && !targetClass.equals("")){
					filter.setAttribute("targetClass", targetClass);
				}	
				if(attrName!=null && !attrName.equals("")){
					filter.setAttribute("attrName", attrName);
				}
				if(eq!=null && !eq.equals("")){
					filter.setAttribute("eq", eq);
				}
				if(neq!=null && !neq.equals("")){
					filter.setAttribute("neq", neq);
				}
				if(lt!=null && !lt.equals("")){
					filter.setAttribute("lt", lt);
				}
				if(gt!=null && !gt.equals("")){
					filter.setAttribute("gt", gt);
				}
				if(lteq!=null && !lteq.equals("")){
					filter.setAttribute("lteq", lteq);
				}
				if(gteq!=null && !gteq.equals("")){
					filter.setAttribute("gteq", gteq);
				}
				if(contains!=null && !contains.equals("")){
					filter.setAttribute("contains", contains);
				}
				
				return filter;
			}else{
				// Syntax Error
				return null;
			}
		}
		
		return null;
	}
}
