package org.livee.data.query;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.util.Collection;
import java.util.Map;

import org.livee.data.expr.AbstractData;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class QueryData implements AbstractData {

	String m_Domain;
	String m_Name;
	//String m_Value;
	
	java.util.Collection m_FilterCol;
	
	protected QueryData(){
		m_FilterCol=new java.util.ArrayList();
	}

	public void append(AbstractData child) {
		if(child==null){
			return ;
		}
		if(child.getDataType().equals("Filter")){
			m_FilterCol.add(child);
		}
	}

	public String getAttribute(String name) {
		if(name==null){
			return null;
		}
		if(name.equals("name")){
			return m_Name;
		}else if(name.equals("domain")){
			return m_Domain;
		}
		return null;
	}

	public Map getAttributes() {
		Map attrs=new java.util.Hashtable();

		if(m_Name!=null){
			attrs.put("name",m_Name);
		}
		if(m_Domain!=null){
			attrs.put("domain", m_Domain);
		}
		return attrs;
	}

	public Collection getChildren() {
		return m_FilterCol;
	}

	public String getDataType() {
		return "Query";
	}

	public String getTextContent() {
		return null;
	}

	public Element getXml(Document doc) {
		
		String ns="http://live-e.org/Query/2007/03/";
		Element query=doc.createElementNS(ns, "query");
		
		if(m_Name!=null){
			query.setAttribute("name",m_Name);
		}
		if(m_Domain!=null){
			query.setAttribute("domain", m_Domain);
		}

		
		java.util.Iterator itr=m_FilterCol.iterator();
		while(itr.hasNext()){
			Object obj=itr.next();
			if(obj!=null){
				AbstractData lc=(AbstractData)obj;
				query.appendChild(lc.getXml(doc));
			}
		}
		
		return query;
	}

	public void remove(AbstractData child) {}

	public void removeAttribute(String name) {

		if(name==null){
			return ;
		}
		
		if(name.equals("name")){
			m_Name=null;
		}else if(name.equals("domain")){
			m_Domain=null;
		}	
	}

	public void setAttribute(String name, String value) {
		
		if(name==null){
			return ;
		}
		
		if(name.equals("name")){
			m_Name=value;
		}else if(name.equals("domain")){
			m_Domain=value;
		}
	}

	public void setTextContent(String text) {
	}
	
	public String generateSQL(){
		
		if(m_FilterCol.size()>0){
			StringBuilder sb=new StringBuilder("SELECT name,server FROM admin.neighbor WHERE (type=\'child\' OR type=\'colleague\') AND name IN (SELECT site AS name FROM admin.combined WHERE");
			java.util.Iterator itr=m_FilterCol.iterator();
			while(itr.hasNext()){
				FilterData filter=(FilterData)itr.next();
				String filter_predicate=filter.generatePredicateElement("combined");
				sb.append(filter_predicate);
				sb.append(" AND");
			}
			int length=sb.length();
			sb.delete(length-4, length);
			sb.append(");");
			return sb.toString();
		}else{
			return "SELECT name,server FROM admin.neighbor WHERE type=\'child\' OR type=\'colleague\';";
		}
	}
}
