package org.livee.data.query.resolve;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.expr.AbstractData;
import org.livee.data.expr.AbstractParser;
import org.livee.data.expr.ParserFactory;
import org.livee.data.query.DataFactory;
import org.livee.data.soap.client.Admin200703Service;
import org.livee.data.soap.client.Admin200703ServiceLocator;
import org.livee.data.soap.client.Admin200703_PortType;
import org.livee.data.soap.client.DataProvider200703Service;
import org.livee.data.soap.client.DataProvider200703ServiceLocator;
import org.livee.data.soap.client.DataProvider200703_PortType;
import org.livee.util.Xml;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class GetProfileAllRetriever implements IRetriever {

	String m_TimeZone;
	String m_Locale;
	String m_QueryString;
	AbstractData m_Query;
	
	public GetProfileAllRetriever(String tz,String lang){

		m_TimeZone=tz;
		m_Locale=lang;
		
		DataFactory df=DataFactory.getInstance();
		AbstractData query=df.create("query");

		query.setAttribute("domain", "admin");
		query.setAttribute("name", "*");

		Document doc=org.livee.util.Xml.newDocument();
		String query_str=Xml.toString(query.getXml(doc));
		
		m_Query=query;
		m_QueryString=query_str;
	}
	
	public String generateKey() {
		return "profileAll,"+m_TimeZone+","+m_Locale;
	}

	public AbstractData getQuery() {
		return m_Query;
	}

	public String getQueryString() {
		return m_QueryString;
	}

	public String getRetriverName() {
		return "GetProfileAll";
	}

	public AbstractData retrieveSite(AbstractData adminSite) {

		if(adminSite==null){
			return null;
		}
		
		java.util.Iterator itr=adminSite.getChildren().iterator();
		while(itr.hasNext()){
			AbstractData adminServer=(AbstractData)itr.next();
			String service=adminServer.getAttribute("service");
			String url=adminServer.getAttribute("url");
			String linkStatus=adminServer.getAttribute("linkStatus");
			
			if(linkStatus==null || (linkStatus!=null && linkStatus.equals("up"))){
				if(service.equals("Admin200703")){

					String profile=null;
					String dataProviderUrl=null;
					
					// Try to get latest data (by implicit rule)
					int r=url.lastIndexOf('/');
					if(r>0){
						dataProviderUrl=url.substring(0,r)+"/DataProvider200703";
					}
					try{
						// Request to GetLatestDataAll
						DataProvider200703Service dp=new DataProvider200703ServiceLocator();
						DataProvider200703_PortType srv=dp.getDataProvider200703(new java.net.URL(dataProviderUrl));
						
						profile=srv.getProfileAll(m_TimeZone,m_Locale);
						
					}catch(Exception e){
						
					}
					
					if(profile==null){
						// Try Again (with explicit resolution of the service)
						try{
							Admin200703Service admin=new Admin200703ServiceLocator();
							Admin200703_PortType srv=admin.getAdmin200703(new java.net.URL(url));
							dataProviderUrl=srv.queryService("DataProvider200703");
							
							// TODO Request to GetLatestDataAll
							DataProvider200703Service dp=new DataProvider200703ServiceLocator();
							DataProvider200703_PortType dpsrv=dp.getDataProvider200703(new java.net.URL(dataProviderUrl));
							
							profile=dpsrv.getProfileAll(m_TimeZone,m_Locale);
													
						}catch(Exception e){
							e.printStackTrace();
						}
					}
					
					if(profile!=null){
						try{
							Document doc=Xml.parseXml(profile);
							Element xml=doc.getDocumentElement();
							ParserFactory pf=ParserFactory.getInstance();
							AbstractParser collection_parser=pf.get("collection");
						
							AbstractData collection=collection_parser.parse(xml);
							return collection;
						}
						catch(Exception e){
							e.printStackTrace();	
						}
					}
				}
			}
		}
		return null;
	}

}
