package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.db.IDbCommand;
import org.livee.data.expr.AbstractData;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class DumpArchiveDataCommand implements IDbCommand {

	boolean m_bSucceeded;
	String m_Start;
	String m_End;
	AbstractData m_DataManagerReplication;
	
	public DumpArchiveDataCommand(String start,String end){
		m_bSucceeded=false;
		m_Start=start;
		m_End=end;
		m_DataManagerReplication=null;
	}
	
	public boolean succeeded(){
		return m_bSucceeded;
	}
	
	public AbstractData getDataManagerReplication(){
		return m_DataManagerReplication;		
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
		
		m_DataManagerReplication=DataFactory.getInstance().create("replicateDataManager");
		
		GetReplicateRawArchiveCollectionCommand rawArchiveCollection_command=new GetReplicateRawArchiveCollectionCommand(m_Start,m_End);
		GetReplicateAggHourCollectionCommand aggHourCol_command=new GetReplicateAggHourCollectionCommand(m_Start,m_End);
		GetReplicateAggDayCollectionCommand aggDayCol_command=new GetReplicateAggDayCollectionCommand(m_Start,m_End);
		GetReplicateAggMonthCollectionCommand aggMonthCol_command=new GetReplicateAggMonthCollectionCommand(m_Start,m_End);
		
		m_bSucceeded=true;
		try{
			rawArchiveCollection_command.execCommand(connection, conf);
			if(rawArchiveCollection_command.succeeded()){
				m_DataManagerReplication.append(rawArchiveCollection_command.getRawArchiveCol());
			}else{
				m_bSucceeded=false;
			}

			aggHourCol_command.execCommand(connection, conf);
			if(aggHourCol_command.succeeded()){
				m_DataManagerReplication.append(aggHourCol_command.getAggregationCol());
			}else{
				m_bSucceeded=false;
			}
			
			aggDayCol_command.execCommand(connection, conf);
			if(aggDayCol_command.succeeded()){
				m_DataManagerReplication.append(aggDayCol_command.getAggregationCol());
			}else{
				m_bSucceeded=false;
			}

			aggMonthCol_command.execCommand(connection, conf);
			if(aggMonthCol_command.succeeded()){
				m_DataManagerReplication.append(aggMonthCol_command.getAggregationCol());
			}else{
				m_bSucceeded=false;
			}
		}
		catch(Exception e){
			m_bSucceeded=false;
			throw e;
		}
	}

	public String getCommandName() {
		return "DumpArchiveData";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
