package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.db.IDbCommand;
import org.livee.data.expr.AbstractData;
import org.livee.data.replicate.DataFactory;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class DumpProfileCommand implements IDbCommand {

	boolean m_bSucceeded;
	String m_Start;
	String m_End;
	AbstractData m_DataManagerReplication;
	
	public DumpProfileCommand(String start,String end){
		m_bSucceeded=false;
		m_Start=start;
		m_End=end;
		m_DataManagerReplication=null;
	}
	
	public boolean succeeded(){
		return m_bSucceeded;
	}
	
	public AbstractData getDataManagerReplication(){
		return m_DataManagerReplication;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
		
		
		m_DataManagerReplication=DataFactory.getInstance().create("replicateDataManager");
		
		m_bSucceeded=true;
		
		try{
			GetReplicateSensorCombinedCollectionCommand combinedCollection_command=new GetReplicateSensorCombinedCollectionCommand();
			combinedCollection_command.execCommand(connection, conf);
			if(combinedCollection_command.succeeded()){
				m_DataManagerReplication.append(combinedCollection_command.getSensorCombinedCol());
			}else{
				m_bSucceeded=false;
			}

			GetReplicateSensorElementCollectionCommand elementCollection_command=new GetReplicateSensorElementCollectionCommand();
			elementCollection_command.execCommand(connection, conf);
			if(elementCollection_command.succeeded()){
				m_DataManagerReplication.append(elementCollection_command.getSensorElementCol());
			}else{
				m_bSucceeded=false;
			}		
			GetReplicateProfileCombinedArchiveCommand combinedArchive_command=new GetReplicateProfileCombinedArchiveCommand(m_Start,m_End);
			combinedArchive_command.execCommand(connection, conf);
			if(combinedArchive_command.succeeded()){
				m_DataManagerReplication.append(combinedArchive_command.getProfileCombinedArchive());
			}else{
				m_bSucceeded=false;
			}
			
			GetReplicateProfileElementArchiveCommand elementArchive_command= new GetReplicateProfileElementArchiveCommand(m_Start,m_End);
			elementArchive_command.execCommand(connection, conf);
			if(elementArchive_command.succeeded()){
				m_DataManagerReplication.append(elementArchive_command.getProfileElementArchive());
			}else{
				m_bSucceeded=false;
			}
		}
		catch(Exception e){
			m_bSucceeded=true;
			throw e;
		}
	}

	public String getCommandName() {
		return "DumpProfile";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
