package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.db.IDbCommand;
import org.livee.data.exception.LiveEException;
import org.livee.data.expr.AbstractData;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class GetReplicateSensorCombinedCollectionCommand implements IDbCommand {

	AbstractData m_SensorCombinedCol;
	boolean m_bSucceeded;
	
	public GetReplicateSensorCombinedCollectionCommand(){
		m_bSucceeded=false;
	}
	
	boolean succeeded(){
		return m_bSucceeded;
	}
	
	public AbstractData getSensorCombinedCol(){
		return m_SensorCombinedCol;
	}

	public void execCommand(Connection connection, Map conf) throws Exception {
	
		java.sql.Statement stmt=null;
		DataFactory df=DataFactory.getInstance();
		m_SensorCombinedCol=df.create("replicateSensorCombinedCollection");
		try{
			stmt=connection.createStatement();
			String sql="SELECT id,passwd,working,ownerName,ownerMailAddress FROM sensor.combined;";
			java.sql.ResultSet rset=stmt.executeQuery(sql);
			
			while(rset.next()){
				String id=rset.getString(1);
				String passwd=rset.getString(2);
				String working="false";
				if(rset.getBoolean(3)){
					working="true";
				}
				String ownerName=rset.getString(4);
				String ownerMailAddress=rset.getString(5);
				
				AbstractData sensorCombined=df.create("replicateSensorCombined");
				sensorCombined.setAttribute("id", id);
				sensorCombined.setAttribute("passwd", passwd);
				sensorCombined.setAttribute("working",working);
				sensorCombined.setAttribute("ownerName",ownerName);
				sensorCombined.setAttribute("ownerMailAddress", ownerMailAddress);
				
				m_SensorCombinedCol.append(sensorCombined);
			}
			
			m_bSucceeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}
	}

	public String getCommandName() {
		return "GetReplicateSensorCombinedCollection";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
