package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.util.Collection;
import java.util.Map;

import org.livee.data.exception.LiveEException;
import org.livee.data.exception.OutOfMaxSizeLiveEException;
import org.livee.data.expr.AbstractData;
import org.livee.data.tools.LogManager;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class RawLatestCollectionReplicationData implements AbstractData {

	// Size Restriction System
	private static boolean m_bSizeBorderLoaded=false;
	private static int m_WarningBorder;
	private static int m_ErrorBorder;
	private static void loadSizeBorder(){
		org.livee.data.tools.ConfigManager config=org.livee.data.tools.ConfigManager.getInstance();
		int warning=config.getIntParameter("/c:liveeConfiguration/c:restriction/c:replication/c:maxRawLatestollectionCount/c:warning/text()");
		int error=config.getIntParameter("/c:liveeConfiguration/c:restriction/c:replication/c:maxRawLatestCollectionCount/c:error/text()");
		if(warning>0){
			m_WarningBorder=warning;
		}else{
			m_WarningBorder=1024;  // recommended default 1024 
		}
		if(error>0){
			m_ErrorBorder=error;
		}else{
			m_ErrorBorder=2048; // recommended default 2048
		}
		m_bSizeBorderLoaded=true;
	}
	
	
	java.util.Collection m_RawLatestCol;
	
	protected RawLatestCollectionReplicationData(){
		m_RawLatestCol=new java.util.ArrayList();
	}

	public void append(AbstractData child) throws LiveEException {
		if(child==null){
			return ;
		}
		if(child.getDataType().equals("RawLatestReplication")){
			
			int size=m_RawLatestCol.size();
			if(m_bSizeBorderLoaded==false){
				loadSizeBorder();
			}
			if(size>m_WarningBorder){
				if(size>m_ErrorBorder){
					// error and exception
					String errorMessage= "The size of rawLatest exceeded error level("+m_ErrorBorder+"): "+size;
					LogManager.postError("restriction.replication.maxRawLatestCollectionCount.error",errorMessage);
					//throw new Exception(errorMessage); 
					throw new OutOfMaxSizeLiveEException("rawLatestCollection",m_ErrorBorder);
				}else{
					// warning
					LogManager.postWarning("restriction.replication.maxRawLatestCollectionCount.warning", "The size of rawLatest exceeded warning level("+m_WarningBorder+"): "+size);
				}
			}

			m_RawLatestCol.add(child);
		}
	}

	public String getAttribute(String name) {
		return null;
	}

	public Map getAttributes() {
		return null;
	}

	public Collection getChildren() {
		return m_RawLatestCol;
	}

	public String getDataType() {
		return "RawLatestCollectionReplication";
	}

	public String getTextContent() {
		return null;
	}

	public Element getXml(Document doc) {
		
		String ns="http://live-e.org/DataManager/2007/03/";
		Element collection=doc.createElementNS(ns, "rawLatestCollection");
		
		java.util.Iterator itr=m_RawLatestCol.iterator();
		while(itr.hasNext()){
			Object obj=itr.next();
			if(obj!=null){
				AbstractData lc=(AbstractData)obj;
				collection.appendChild(lc.getXml(doc));
			}
		}
		
		return collection;
	}

	public void remove(AbstractData child) {}

	public void removeAttribute(String name) {}

	public void setAttribute(String name, String value) {}

	public void setTextContent(String text) {}
}
