package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.db.IDbCommand;
import org.livee.data.expr.AbstractData;
import org.livee.util.W3CTimestamp;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class SetReplicateProfileCombinedArchiveCommand implements IDbCommand {
	
	AbstractData m_ProfileCombinedCol;
	boolean m_bSucceeded;
	
	public SetReplicateProfileCombinedArchiveCommand(AbstractData profileCombinedArchive){
		m_bSucceeded=false;
		m_ProfileCombinedCol=profileCombinedArchive;
	}
	
	boolean succeeded(){
		return m_bSucceeded;
	}
	
	
	public void execCommand(Connection connection, Map conf) throws Exception {
	
		java.sql.Statement stmt=null;
		
		try{
			stmt=connection.createStatement();
			
			java.util.Iterator itr=m_ProfileCombinedCol.getChildren().iterator();
			while(itr.hasNext()){
				AbstractData profileCombined=(AbstractData)itr.next();
				if(profileCombined==null){ continue; }
				String id=profileCombined.getAttribute("id");
				String created=profileCombined.getAttribute("created");
				String expired=profileCombined.getAttribute("expired");
				String latitude=profileCombined.getAttribute("latitude");
				String longitude=profileCombined.getAttribute("longitude");
				String attrs=profileCombined.getAttribute("attrs");
				
				if(id==null || created==null){
					continue;
				}
				
				StringBuilder sb=new StringBuilder("INSERT INTO profile.combined_archive (id, created, expired, latitude, longitude, attrs) VALUES(\'");
				sb.append(id);
				sb.append("\',\'");
				sb.append(created);
				sb.append("\',");
				if(expired!=null){
					sb.append("\'");
					sb.append(expired);
					sb.append("\',");
				}else{
					sb.append("null,");
				}
				
				if(latitude!=null){
					sb.append(latitude);
					sb.append(",");
				}else{
					sb.append("null,");
				}
				
				if(longitude!=null){
					sb.append(longitude);
					sb.append(",");
				}else{
					sb.append("null,");
				}
				
				if(attrs!=null){
					sb.append("\'");
					sb.append(attrs);
					sb.append("\');");
				}else{
					sb.append("\'{}\');");
				}
				
				String sql=sb.toString();
				try{
					stmt.execute(sql);
				}
				catch(Exception e){
					//System.err.println(sql);
					//e.printStackTrace();
					//m_bSucceeded=false;
				}
			}
			m_bSucceeded=true;
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}
	}


	public String getCommandName() {
		return "SetReplicateProfileCombinedArchive";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
