package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.db.IDbCommand;
import org.livee.data.expr.AbstractData;
import org.livee.data.replicate.DataFactory;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class SetReplicateRawLatestCollectionCommand implements IDbCommand {

	AbstractData m_RawLatestCol;
	boolean m_bSucceeded;
	
	public SetReplicateRawLatestCollectionCommand(AbstractData rawLatestCol){
		m_RawLatestCol=rawLatestCol;
		m_bSucceeded=false;
	}
	
	boolean succeeded(){
		return m_bSucceeded;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
	
		java.sql.Statement stmt=null;
		
		try{
			stmt=connection.createStatement();
			m_bSucceeded=true;
			
			java.util.Iterator itr=m_RawLatestCol.getChildren().iterator();
			while(itr.hasNext()){
				AbstractData raw=(AbstractData)itr.next();
				if(raw==null){ continue; }
				String id=raw.getAttribute("id");
				String time=raw.getAttribute("time");
				String attrs=raw.getAttribute("attrs");
				String value=raw.getAttribute("value");
				String sValue=raw.getAttribute("sValue");
				
				if(id==null || time==null){
					continue;
				}
				
				StringBuilder update=new StringBuilder("UPDATE raw.latest SET time=\'");
				update.append(time);
				update.append("\', ");
				
				update.append("attrs=");
				if(attrs!=null){
					update.append("\'");
					update.append(attrs);
					update.append("\', ");
				}else{
					update.append("\'{}\', ");
				}
				
				update.append("value=");
				if(value!=null){
					update.append(value);
					update.append(", ");
				}else{
					update.append("null, ");
				}
				
				update.append("sValue=");
				if(sValue!=null){
					update.append("\'");
					update.append(sValue);
					update.append("\' ");
				}else{
					update.append("null ");
				}
				update.append("WHERE id=\'");
				update.append(id);
				update.append("\';");
				
				String sql=update.toString();
				//System.out.println(sql);
				
				if(stmt.executeUpdate(sql)==0){
					// No entry to update ==> INSERT
					StringBuilder insert=new StringBuilder("INSERT INTO raw.latest (id,time,attrs,value,sValue) VALUES (\'");
					insert.append(id);
					insert.append("\',\'");
					insert.append(time);
					if(attrs!=null){
						insert.append("\',\'");
						insert.append(attrs);
						insert.append("\',");
					}else{
						insert.append("\',\'{}\',");
					}
					
					if(value!=null){
						insert.append(value);
					}else{
						insert.append("null");
					}
					
					if(sValue!=null){
						insert.append(",\'");
						insert.append(sValue);
						insert.append("\');");
					}else{
						insert.append(",null);");
					}

					sql=insert.toString();
					try{
						stmt.execute(sql);
					}
					catch(Exception e){
						//System.err.println(sql);
						//e.printStackTrace();
						m_bSucceeded=false;
					}

				}
			}
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}
	}

	public String getCommandName() {
		return "SetReplicateRawLatestCollection";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
