package org.livee.data.replicate;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.sql.Connection;
import java.util.Map;

import org.livee.data.db.IDbCommand;
import org.livee.data.expr.AbstractData;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class SetReplicateSensorElementCollectionCommand implements IDbCommand {

	AbstractData m_SensorElementCol;
	boolean m_bSucceeded;
	
	public SetReplicateSensorElementCollectionCommand(AbstractData sensorElementCol){
		m_SensorElementCol=sensorElementCol;
		m_bSucceeded=false;
	}
	
	boolean succeeded(){
		return m_bSucceeded;
	}
	
	public void execCommand(Connection connection, Map conf) throws Exception {
	
		java.sql.Statement stmt=null;
		
		try{
			stmt=connection.createStatement();
			m_bSucceeded=true;
			
			java.util.Iterator itr=m_SensorElementCol.getChildren().iterator();
			while(itr.hasNext()){
				AbstractData sensorElement=(AbstractData)itr.next();
				if(sensorElement==null){ continue; }
				String id=sensorElement.getAttribute("id");
				String combined_id=sensorElement.getAttribute("combined_id");
				String working=sensorElement.getAttribute("working");
				
				if(id==null || combined_id==null){
					continue;
				}
				
				// Try update first
				StringBuilder update=new StringBuilder("UPDATE sensor.element SET combined_id=\'");
				update.append(combined_id);
				update.append("\', ");
				
				if(working!=null && working.startsWith("t")){
					update.append("working=\'t\' ");
				}else{
					update.append("working=\'f\' ");
				}
				update.append("WHERE id=\'");
				update.append(id);
				update.append("\';");
				
				String sql=update.toString();
				
				if(stmt.executeUpdate(sql)==0){
					// No entry to update ==> INSERT action
					StringBuilder insert=new StringBuilder("INSERT INTO sensor.element (id,combined_id,working) VALUES (\'");
					insert.append(id);
					insert.append("\',\'");
					insert.append(combined_id);
					if(working!=null && working.startsWith("t")){
						insert.append("\',\'t\');");
					}else{
						insert.append("\',\'f\');");
					}
					
					sql=insert.toString();
					try{
						stmt.execute(sql);
					}
					catch(Exception e){
						//System.err.println(sql);
						//e.printStackTrace();
						m_bSucceeded=false;
					}
				}
				
				
			}
		}
		catch(Exception e){
			throw e;
		}
		finally{
			stmt.close();
		}
	}


	public String getCommandName() {
		return "SetReplicateSensorElementCollection";
	}

	public short getImplVersion() {
		return 0x0100;
	}

}
