package org.livee.data.srv;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.data.db.CommandFactory;
import org.livee.data.db.DatabaseAdapter;
import org.livee.data.exception.InternalLiveEException;
import org.livee.data.exception.LiveEException;
import org.livee.data.expr.AbstractData;
import org.livee.data.expr.AbstractParser;
import org.livee.data.expr.ParserFactory;
import org.livee.data.tools.LogManager;
import org.livee.data.db.UploadElementCommand;
import org.livee.data.db.UploadCombinedCommand;


import org.w3c.dom.Document;
import org.livee.util.Base64Coder;
import org.livee.util.Xml;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class DataUpload200703 {
	
	public String uploadElement(String xml){
	
		LogManager.postNormal("service.DataUpload200703.uploadElement.begin",null);
		
		AbstractParser elementParser=ParserFactory.getInstance().get("element");
		try{
			Document doc=Xml.parseXml(xml);
			AbstractData element=elementParser.parse(doc.getDocumentElement());
			
			String passwd=Base64Coder.decodeString(element.getAttribute("authorization"));
			
			element.removeAttribute("xmlns");
			element.removeAttribute("authorization");
			
			UploadElementCommand command=CommandFactory.createUploadElementCommand(element, passwd);
			
			DatabaseAdapter db = DatabaseAdapter.getInstance();

			db.postCommand(command);

			if (command.succeeded()){
				LogManager.postNormal("service.DataUpload200703.uploadElement.end",null);
				return "<true />";
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataUpload200703.uploadElement.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataUpload200703.uploadElement.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataUpload200703.uploadElement.error", "unknown");
		return exp.getMessage();
	}
	
	public String uploadCombined(String xml){
		
		LogManager.postNormal("service.DataUpload200703.uploadCombined.begin",null);
		
		AbstractParser combinedParser=ParserFactory.getInstance().get("combined");
		
		try{
			Document doc=Xml.parseXml(xml);
			AbstractData combined=combinedParser.parse(doc.getDocumentElement());
			
			String passwd=Base64Coder.decodeString(combined.getAttribute("authorization"));
			
			combined.removeAttribute("xmlns");
			combined.removeAttribute("authorization");
			
			UploadCombinedCommand command=CommandFactory.createUploadCombinedCommand(combined, passwd);
			
			DatabaseAdapter db = DatabaseAdapter.getInstance();

			db.postCommand(command);
			if (command.succeeded()){
				LogManager.postNormal("service.DataUpload200703.uploadCombined.end",null);
				return "<true />";
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataUpload200703.uploadCombined.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataUpload200703.uploadCombined.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataUpload200703.uploadCombined.error", "unknown");
		return exp.getMessage();
	}
	
	public String uploadCollection(String xml){
	
		LogManager.postNormal("service.DataUpload200703.uploadCollection.begin",null);
		
		AbstractParser collParser=ParserFactory.getInstance().get("collection");
		
		try{
			Document doc=Xml.parseXml(xml);
			AbstractData collection=collParser.parse(doc.getDocumentElement());
			
			boolean authorization=true;
			AbstractData combined;
			DatabaseAdapter db = DatabaseAdapter.getInstance();

			java.util.Iterator itr=collection.getChildren().iterator();
			while(itr.hasNext()){
				combined=(AbstractData)itr.next();

				String passwd=Base64Coder.decodeString(combined.getAttribute("authorization"));
				combined.removeAttribute("authorization");
				
				UploadCombinedCommand command=CommandFactory.createUploadCombinedCommand(combined, passwd);
				db.postCommand(command);
				
				if(!command.succeeded()){
					authorization=false;
				}
			}
			
			if (authorization){
				LogManager.postNormal("service.DataUpload200703.uploadCollection.end",null);
				return "<true />";
			}
		}
		catch(LiveEException e){
			String msg=e.getMessage();
			LogManager.postError("service.DataUpload200703.uploadCollection.error", msg);
			return msg;
		}
		catch(Exception e){
			LiveEException exp=new InternalLiveEException(e.getMessage());
			LogManager.postError("service.DataUpload200703.uploadCollection.error", e.getMessage());
			return exp.getMessage();
		}

		// return error
		LiveEException exp=new InternalLiveEException("unknown");
		LogManager.postError("service.DataUpload200703.uploadCollection.error", "unknown");
		return exp.getMessage();
		
	}
}
