package org.livee.data.tools;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class ConfigManager {

	private static ConfigManager m_Singleton=null;
	private java.util.Map m_ConfigCache;
	private Document m_Document;
	private Element m_ConfigRoot;
	private javax.xml.xpath.XPath m_XPath;
	
	ConfigManager() {	
		// get file path from LIVEE_HOME
		String home=System.getenv("LIVEE_HOME");
		
		String confPath=null;
		if(home!=null && !home.equals("")){
			if(home.endsWith("/")){
				confPath=home+"conf/livee_config.xml";
			}else{
				confPath=home+"/conf/livee_config.xml";
			}
			// System.out.println("LIVEE_HOME: "+home);
		}else{
			confPath="conf/livee_config.xml";
			System.err.println("LIVEE_HOME is not set.");
			System.exit(1);
		}
		
		// if error, throw Error and m_Singleton=null; 
		m_Document=org.livee.util.Xml.parseXmlFile(confPath);
		m_ConfigRoot=m_Document.getDocumentElement();
		
		XPathFactory xp_factory=XPathFactory.newInstance();
		m_XPath=xp_factory.newXPath();
		m_XPath.setNamespaceContext(new ConfigNamespaceContext());
		
		m_ConfigCache=new java.util.Hashtable();
		m_Singleton=this;
	}
	
	public static ConfigManager getInstance() {
		if(m_Singleton==null){
			new ConfigManager();
		}
		return m_Singleton;
	}
	
	private void setCache(String key,Object o){
		if(m_ConfigCache.size()<1000){
			m_ConfigCache.put(key, o);
		}
	}
	
	private Object getCache(String key){
		return m_ConfigCache.get(key);
	}
	
	public String getStringParameter(String key){
		Object o=getCache(key);
		if(o!=null){
			return (String)o;
		}
		
		String value=null;
		try{
			value=m_XPath.evaluate(key, m_ConfigRoot);
		}
		catch(Exception e){
			e.printStackTrace();
			return null;
		}
		setCache(key,value);
		return value;
	}
	
	public int getIntParameter(String key){
		Object o=getCache(key);
		if(o!=null){
			return ((Integer)o).intValue();
		}
		
		Integer value=null;
		try{
			String strValue=m_XPath.evaluate(key, m_ConfigRoot);
			value=new Integer(strValue);
		}
		catch(Exception e){
			return -1;
		}
		setCache(key,value);
		return value.intValue();
	}
	
	public double getDoubleParameter(String key){
		Object o=getCache(key);
		if(o!=null){
			return ((Double)o).doubleValue();
		}
		
		Double value=null;
		try{
			String strValue=m_XPath.evaluate(key, m_ConfigRoot);
			value=new Double(strValue);
		}
		catch(Exception e){
			return 0.0;
		}
		setCache(key,value);
		return value.doubleValue();
	}
	
	public boolean getBooleanParameter(String key){
		Object o=getCache(key);
		if(o!=null){
			return ((Boolean)o).booleanValue();
		}
		
		Boolean value=null;
		try{
			String strValue=m_XPath.evaluate(key, m_ConfigRoot);
			value=new Boolean(strValue);
		}
		catch(Exception e){
			return false;
		}
		setCache(key,value);
		return value.booleanValue();
	}
	
	public org.w3c.dom.Node getNodeParameter(String key){
		
		Object o=getCache(key);
		if(o!=null){
			return (Node)o;
		}
		
		org.w3c.dom.Node value=null;
		try{
			value=(Node)m_XPath.evaluate(key, m_ConfigRoot,XPathConstants.NODE);
		}
		catch(Exception e){
			return null;
		}
		if(value==null){
			System.out.println("NULL!!");
		}
		setCache(key,value);
		return value;
	}
}
