package org.livee.data.tools;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import org.livee.util.W3CTimestamp;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class LogManager {

	private static LogManager m_Singleton=null;
	private static java.util.TimeZone m_TZ=null;
	private NormalLogWriter m_NormalLogWriter;
	private WarningLogWriter m_WarningLogWriter;
	private ErrorLogWriter m_ErrorLogWriter;
	
	private LogMessageBuffer m_NormalLogBuffer;
	private LogMessageBuffer m_WarningLogBuffer;
	private LogMessageBuffer m_ErrorLogBuffer;
	
	
	private LogManager(){
		m_NormalLogBuffer=new LogMessageBuffer();
		m_WarningLogBuffer=new LogMessageBuffer();
		m_ErrorLogBuffer=new LogMessageBuffer();
		
		m_NormalLogWriter=new NormalLogWriter(m_NormalLogBuffer);
		m_WarningLogWriter=new WarningLogWriter(m_WarningLogBuffer);
		m_ErrorLogWriter=new ErrorLogWriter(m_ErrorLogBuffer);
		m_Singleton=this;
	}
	
	private static LogManager getInstance(){
		if(m_Singleton==null){
			new LogManager();
		}
		return m_Singleton;
	}
	
	private static String getCurTime(){
		if(m_TZ==null){ m_TZ=java.util.TimeZone.getDefault(); }
		return W3CTimestamp.toString(System.currentTimeMillis(), m_TZ);
	}
	
	public static void postNormal(String log_id,String msg){
		LogMessage logmsg=new LogMessage(log_id,msg,getCurTime());		
		getInstance().m_NormalLogBuffer.postMessage(logmsg);
	}
	
	public static void postWarning(String log_id,String msg){
		LogMessage logmsg=new LogMessage(log_id,msg,getCurTime());
		getInstance().m_WarningLogBuffer.postMessage(logmsg);
	}
	
	public static void postError(String log_id,String msg){
		LogMessage logmsg=new LogMessage(log_id,msg,getCurTime());
		getInstance().m_ErrorLogBuffer.postMessage(logmsg);
	}
}
