package org.livee.data.tools;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.util.LinkedList;
import java.util.Queue;

/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class LogMessageBuffer {

	Queue m_MessageQueue;
	
	/**
	 * 
	 */
	LogMessageBuffer(){
		m_MessageQueue=new LinkedList();
	}
	
	/**
	 * synchronized 
	 * @param msg
	 */
	public synchronized void postMessage(LogMessage msg){
		m_MessageQueue.offer(msg);
		notifyAll();
	}
	
	/**
	 * 
	 * @return
	 */
	public synchronized LogMessage peekMessage(){
		return (LogMessage)m_MessageQueue.peek();
	}
	
	/**
	 * 
	 * @return
	 */
	public synchronized LogMessage popMessage(){
		return (LogMessage)m_MessageQueue.poll();
	}
	
	/**
	 * 
	 * @param timeout millisecond
	 * @return
	 */
	public synchronized LogMessage acceptMessage(long timeout){		
		if(m_MessageQueue.size()>0){
			return (LogMessage)m_MessageQueue.poll();
		}
		try{
			wait(timeout);
		}
		catch(Exception e){
			e.printStackTrace();
		}
		if(m_MessageQueue.size()>0){
			return (LogMessage)m_MessageQueue.poll();
		}
		return null;
	}
	
	/**
	 * 
	 * @return
	 */
	public synchronized LogMessage acceptMessage(){
		
		if(m_MessageQueue.size()>0){
			return (LogMessage)m_MessageQueue.poll();
		}
		try{
			wait();
		}
		catch(Exception e){
			e.printStackTrace();
		}
		if(m_MessageQueue.size()>0){
			return (LogMessage)m_MessageQueue.poll();
		}
		return null;
	}
	
	/**
	 * @return How many messages are buffered now.
	 */
	public synchronized int count(){
		return m_MessageQueue.size();
	}
}
