package org.livee.data.tools;
/*
 * Copyright (c) 2007 Hideya Ochiai,  All rights reserved.
 * 
 * Permission of redistribution and use in source and binary forms, 
 * with or without modification, are granted, free of charge, to any person 
 * obtaining the copy of this software under the following conditions:
 * 
 *  1. Any copies of this source code must include the above copyright notice,
 *  this permission notice and the following statement without modification 
 *  except possible additions of other copyright notices. 
 * 
 *  2. Redistributions of the binary code must involve the copy of the above 
 *  copyright notice, this permission notice and the following statement 
 *  in documents and/or materials provided with the distribution.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.Date;
import java.util.Properties;

import javax.mail.Message;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;


/**
 * @author Hideya Ochiai, The University of Tokyo
 * Created: before 2008-01-01
 */
public class WarningLogWriter extends Thread {
	protected LogMessageBuffer m_WarningLogQueue;
	private String m_SmtpServer;
	private String m_MailAddress;
	private String m_OutputFilePath;

	public WarningLogWriter(LogMessageBuffer queue){
		m_WarningLogQueue=queue;
		ConfigManager config=ConfigManager.getInstance();
		String smtpServer=config.getStringParameter("/c:liveeConfiguration/c:logging/c:smtpServer/text()");
		String mailAddress=config.getStringParameter("/c:liveeConfiguration/c:logging/c:warning/c:mail/text()");
		String outputFilePath=config.getStringParameter("/c:liveeConfiguration/c:logging/c:warning/c:file/text()");
		String home=System.getenv("LIVEE_HOME");
		
		m_SmtpServer=smtpServer;
		if(mailAddress!=null && mailAddress.contains("@")){
			m_MailAddress=mailAddress;
		}else{
			m_MailAddress=null;
		}
		if(outputFilePath!=null && !outputFilePath.equals("")){
			if(home.endsWith("/")){
				m_OutputFilePath=home+"log/"+outputFilePath;
			}else{
				m_OutputFilePath=home+"/log/"+outputFilePath;
			}
		}else{
			m_OutputFilePath=null;
		}

		start();
	}
	
	public void run(){
		
		while(true){
			LogMessage msg=m_WarningLogQueue.acceptMessage();
			
			try{ Thread.sleep(2000); }catch(Exception e){}
			
			StringBuilder sb=new StringBuilder();
			sb.append(createMessageLine(msg));
			int count=m_WarningLogQueue.count();
			for(int i=0;i<count;i++){
				msg=m_WarningLogQueue.popMessage();
				sb.append(createMessageLine(msg));
			}
			String outputMessage=sb.toString();
			
			if(m_OutputFilePath!=null){
				try{
					BufferedWriter writer=new BufferedWriter(new FileWriter(m_OutputFilePath,true));
					writer.write(outputMessage);
					writer.close();
				}catch(Exception e){
				//	e.printStackTrace();
					// TODO Error log
				}
			}
			if(m_SmtpServer!=null && m_MailAddress!=null){
				try{
				      Properties props = System.getProperties();
				      
				      props.put("mail.smtp.host", m_SmtpServer);
				      Session session=Session.getDefaultInstance(props);
				      MimeMessage mimeMessage=new MimeMessage(session);
				      mimeMessage.setFrom(new InternetAddress("jo2lxq@hongo.wide.ad.jp","Live E! LogManager (NormalLog) Daemon","iso-2022-jp"));
				      mimeMessage.setRecipients(Message.RecipientType.TO,m_MailAddress);
				      mimeMessage.setSubject("Live E! Normal Log Output","iso-2022-jp");
				      mimeMessage.setText(outputMessage,"iso-2022-jp");
				      mimeMessage.setHeader("Content-Type","text/plain");
				      mimeMessage.setSentDate(new Date());
				      Transport.send(mimeMessage);
				}
				catch(Exception e){
					//e.printStackTrace();
					// TODO Error log
				}
			}
		}
	}
	
	private String createMessageLine(LogMessage msg){
		String logID=msg.getLogID();
		String time=msg.getTimestamp();
		String message=msg.getMessage();
		
		if(message==null){
			return time+" "+logID+" "+"\r\n";
		}else{
			return time+" "+logID+" "+message+"\r\n";
		}
	}

}
